@@ -49,6 +49,26 @@ SyntaxDef     DummyPopupFuncSyntax = {
 	NULL,	0
 };
 
+TermDef       DummyTerms[] =
+{
+    {0, NULL, 0, 0, 0}
+};
+
+
+SyntaxDef     ArrangeSyntax = {
+	'\n',
+	'\0',
+	DummyTerms,
+	0,										   /* use default hash size */
+    ' ',
+	"",
+	"\t",
+	"Module:Arrange",
+	"Arrange",
+	"AfterStep module for arranging windows in a tiling or cascading order",
+	NULL,
+	0
+};
 
 
 
@@ -102,6 +122,8 @@ TermDef       WindowBoxTerms[] = {
     {TF_NO_MYNAME_PREPENDING, "MaxLayer", 8,          TT_INTEGER, WINDOWBOX_MaxLayer_ID      , NULL},
     {TF_NO_MYNAME_PREPENDING, "VerticalPriority", 16, TT_FLAG,    WINDOWBOX_VerticalPriority_ID, NULL},
 	{TF_NO_MYNAME_PREPENDING, "ReverseOrder", 12,     TT_FLAG,    WINDOWBOX_ReverseOrder_ID    , NULL},
+	{TF_NO_MYNAME_PREPENDING, "ReverseOrderHorizontal", 22,     TT_FLAG,    WINDOWBOX_ReverseOrderHorizontal_ID    , NULL},
+	{TF_NO_MYNAME_PREPENDING, "ReverseOrderVertical", 20,       TT_FLAG,    WINDOWBOX_ReverseOrderVertical_ID    , NULL},
     {TF_NO_MYNAME_PREPENDING |
 	 TF_SYNTAX_TERMINATOR,    "~WindowBox", 10, TT_FLAG, WINDOWBOX_DONE_ID, NULL},
 	{0, NULL, 0, 0, 0}
@@ -213,6 +235,10 @@ SyntaxDef includeSyntax =
     {TF_NO_MYNAME_PREPENDING, "StartMenuSortMode", 17, TT_UINTEGER, LOOK_StartMenuSortMode_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "DrawMenuBorders", 15, TT_UINTEGER, LOOK_DrawMenuBorders_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "ButtonSize", 10, TT_BOX, LOOK_ButtonSize_ID, NULL}, \
+    {TF_NO_MYNAME_PREPENDING, "ButtonIconSpacing", 17, TT_UINTEGER, LOOK_ButtonIconSpacing_ID, NULL}, \
+    {TF_NO_MYNAME_PREPENDING, "ButtonBevel", 11, TT_FLAG, LOOK_ButtonBevel_ID, &BevelSyntax}, \
+    {TF_NO_MYNAME_PREPENDING, "ButtonAlign", 11, TT_FLAG, LOOK_ButtonAlign_ID, &AlignSyntax}, \
+ 	{TF_NO_MYNAME_PREPENDING, "MinipixmapSize", 14, TT_BOX, LOOK_MinipixmapSize_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "SeparateButtonTitle", 19, TT_FLAG, LOOK_SeparateButtonTitle_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "RubberBand", 10, TT_UINTEGER, LOOK_RubberBand_ID, NULL}, \
     {TF_NO_MYNAME_PREPENDING, "DefaultStyle", 12, TT_QUOTED_TEXT, LOOK_DefaultStyle_ID, NULL}, \
@@ -452,6 +478,7 @@ TermDef FunctionTerms[] =
 };
 
 extern TermDef       WharfTerms[];
+extern TermDef       WharfFolderTerms[];
 
 /**************************************************************************
  * Syntaxes : 
@@ -486,6 +513,7 @@ void LinkAfterStepConfig()
 						PopupTerms,	
 						FunctionTerms,
 						WharfTerms,
+						WharfFolderTerms,
 						NULL } ;
 	while( termsets[i] ) 
 	{
