@@ -42,6 +42,7 @@ TermDef       WinTabsTerms[] = {
     {0, "MaxTabWidth", 11, TT_INTEGER, WINTABS_MaxTabWidth_ID, NULL},
     {0, "MinTabWidth", 11, TT_INTEGER, WINTABS_MinTabWidth_ID, NULL},
     {TF_INDEXED, "Pattern", 7, TT_QUOTED_TEXT, WINTABS_Pattern_ID, NULL},
+    {TF_INDEXED, "ExcludePattern", 14, TT_QUOTED_TEXT, WINTABS_ExcludePattern_ID, NULL},
     {0, "Align", 5, TT_FLAG, WINTABS_Align_ID, &AlignSyntax},
     {0, "Bevel", 5, TT_FLAG, WINTABS_Bevel_ID, &BevelSyntax},
     {0, "FBevel", 6, TT_FLAG, WINTABS_FBevel_ID, &BevelSyntax},
@@ -62,6 +63,10 @@ TermDef       WinTabsTerms[] = {
     {0, "FocusedStyle", 12, TT_TEXT, WINTABS_FocusedStyle_ID, NULL},
     {0, "StickyStyle", 11, TT_TEXT, WINTABS_StickyStyle_ID, NULL},
     {0, "UseSkipList", 11, TT_FLAG, WINTABS_UseSkipList_ID, NULL},
+    {0, "AllDesks", 8, TT_FLAG, WINTABS_AllDesks_ID, NULL},
+    {0, "Title", 5, TT_TEXT, WINTABS_Title_ID, NULL},
+    {0, "IconTitle", 9, TT_TEXT, WINTABS_IconTitle_ID, NULL},
+    {0, "SkipTransients", 14, TT_FLAG, WINTABS_SkipTransients_ID, NULL},
 /* including MyStyles definitions processing */
 	INCLUDE_MYSTYLE,
 
@@ -97,7 +102,8 @@ CreateWinTabsConfig ()
 	config->pattern_type = ASN_Name;
     config->name_aligment = ALIGN_CENTER;
     config->balloon_conf = NULL;
-    config->h_spacing = config->v_spacing = DEFAULT_TBAR_SPACING;
+    config->h_spacing = DEFAULT_TBAR_HSPACING;
+	config->v_spacing = DEFAULT_TBAR_VSPACING;
     config->fbevel = config->ubevel = config->sbevel = DEFAULT_TBAR_HILITE ;
 
 	return config;
@@ -159,6 +165,13 @@ PrintWinTabsConfig (WinTabsConfig * config)
 	fprintf (stderr, "WINTABSConfig.sticky_style = %p;\n", config->sticky_style);
 	if (config->sticky_style)
 		fprintf (stderr, "WINTABSConfig.sticky_style = \"%s\";\n", config->sticky_style);
+	if (config->title)
+		fprintf (stderr, "WINTABSConfig.title = \"%s\";\n", config->title);
+	if (config->icon_title)
+		fprintf (stderr, "WINTABSConfig.icon_title = \"%s\";\n", config->icon_title);
+	fprintf (stderr, "WinTabsConfig.SkipTransients = %s;\n",
+			 get_flags (config->flags, WINTABS_SkipTransients) ? "True" : "False");
+
 }
 
 WinTabsConfig *
@@ -199,6 +212,12 @@ ParseWinTabsOptions (const char *filename, char *myname)
                 case WINTABS_UseSkipList_ID:
                     SET_CONFIG_FLAG (config->flags, config->set_flags, WINTABS_UseSkipList);
                     break;
+                case WINTABS_AllDesks_ID:
+                    SET_CONFIG_FLAG (config->flags, config->set_flags, WINTABS_AllDesks);
+                    break;
+                case WINTABS_SkipTransients_ID:
+                    SET_CONFIG_FLAG (config->flags, config->set_flags, WINTABS_SkipTransients);
+                    break;
                 case WINTABS_Align_ID :
                     set_flags( config->set_flags, WINTABS_Align );
                     config->name_aligment = ParseAlignOptions( pCurr->sub );
@@ -250,7 +269,14 @@ ParseWinTabsOptions (const char *filename, char *myname)
                 case WINTABS_Pattern_ID:
                     config->pattern_type = item.index % ASN_NameTypes;
                     LOCAL_DEBUG_OUT( "old_pattern = %p, new pattern = %s", config->pattern, item.data.string );
-					set_string_value( &(config->pattern), item.data.string, NULL, 0 );
+		set_string_value( &(config->pattern), item.data.string, 
+				  &(config->set_flags), WINTABS_PatternType );
+                    break;
+                case WINTABS_ExcludePattern_ID:
+                    config->exclude_pattern_type = item.index % ASN_NameTypes;
+                    LOCAL_DEBUG_OUT( "old_excl_pattern = %p, new excl_pattern = %s", config->exclude_pattern, item.data.string );
+					set_string_value( &(config->exclude_pattern), item.data.string, 
+				  &(config->set_flags), WINTABS_ExcludePatternType );
                     break;
                 case WINTABS_UnfocusedStyle_ID:
                     REPLACE_STRING (config->unfocused_style, item.data.string);
@@ -293,6 +319,12 @@ ParseWinTabsOptions (const char *filename, char *myname)
                     set_flags( config->set_flags, WINTABS_V_SPACING );
                     config->v_spacing = item.data.integer;
                     break ;
+                case WINTABS_Title_ID:
+					set_string_value( &(config->title), item.data.string, NULL, 0 );
+                    break;
+                case WINTABS_IconTitle_ID:
+					set_string_value( &(config->icon_title), item.data.string, NULL, 0 );
+                    break;
 
                 default:
                     item.ok_to_free = 1;
