@@ -101,7 +101,7 @@ asvisual_empty_XErrorHandler (Display * dpy, XErrorEvent * event)
     return 0;
 }
 /***************************************************************************/
-#ifdef LOCAL_DEBUG
+#if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
 Status
 debug_AllocColor( const char *file, const char *func, int line, ASVisual *asv, Colormap cmap, XColor *pxcol )
 {
@@ -499,6 +499,8 @@ destroy_asvisual( ASVisual *asv, Bool reusable )
 		if( asv->glx_scratch_gc_indirect )
 			glXDestroyContext(dpy, asv->glx_scratch_gc_indirect );
 #endif
+		if( asv->scratch_window ) 
+			XDestroyWindow( asv->dpy, asv->scratch_window );
 
 #endif /*ifndef X_DISPLAY_MISSING */
 		if( !reusable )
@@ -1072,13 +1074,11 @@ create_visual_gc( ASVisual *asv, Window root, unsigned long mask, XGCValues *gcv
 #ifndef X_DISPLAY_MISSING
 	if( asv )
 	{
-		Window scratch_window = 0 ;
 		XGCValues scratch_gcv ;
-		if( (scratch_window = create_visual_window( asv, root, -20, -20, 10, 10, 0, InputOutput, 0, NULL )) != None )
-		{
-			gc = XCreateGC( asv->dpy, scratch_window, mask, gcvalues?gcvalues:&scratch_gcv );
-			XDestroyWindow( asv->dpy, scratch_window );
-		}
+		if( asv->scratch_window == None ) 
+			asv->scratch_window = create_visual_window( asv, root, -20, -20, 10, 10, 0, InputOutput, 0, NULL );
+		if( asv->scratch_window != None )
+			gc = XCreateGC( asv->dpy, asv->scratch_window, gcvalues?mask:0, gcvalues?gcvalues:&scratch_gcv );
 	}
 #endif
 	return gc;
@@ -1090,7 +1090,11 @@ create_visual_pixmap( ASVisual *asv, Window root, unsigned int width, unsigned i
 #ifndef X_DISPLAY_MISSING
 	Pixmap p = None ;
 	if( asv != NULL )
+	{	
+		if( root == None ) 
+			root = RootWindow(asv->dpy,DefaultScreen(asv->dpy));
 		p = XCreatePixmap( asv->dpy, root, MAX(width,(unsigned)1), MAX(height,(unsigned)1), (depth==0)?asv->true_depth:depth );
+	}
 	return p;
 #else
 	return None ;
@@ -1291,7 +1295,7 @@ destroy_xshmimage_segment(ASHashableValue value, void *data)
 	ASXShmImage *img_data = (ASXShmImage*)data ;
 	if( img_data->segment != NULL )
 	{
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : segent to be freed: shminfo = %p, seg = %d ", img_data->segment, img_data->segment->shmid );
+		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : img_data = %p : segent to be freed: shminfo = %p ", img_data, img_data->segment );
 		XShmDetach (dpy, img_data->segment);
 		save_shm_area( img_data->segment->shmaddr, img_data->segment->shmid, img_data->size );
 		free( img_data->segment );
@@ -1300,7 +1304,7 @@ destroy_xshmimage_segment(ASHashableValue value, void *data)
 			free( img_data );
 	}else
 	{
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : segment data is NULL already value = %ld!!", value );
+		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_SEG : img_data = %p : segment data is NULL already value = %ld!!", img_data, value );
 	}
 }
 
@@ -1333,11 +1337,11 @@ destroy_xshmimage_image(ASHashableValue value, void *data)
 			orig_XShmImage_destroy_image( img_data->ximage );
 		else
 			XFree ((char *)img_data->ximage);
-		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_XIM : ximage freed: xim = %p", img_data->ximage);
+		LOCAL_DEBUG_OUT( "XSHMIMAGE> FREE_XIM : ximage freed: img_data = %p, xim = %p", img_data, img_data->ximage);
 		img_data->ximage = NULL ;
 		if( img_data->segment != NULL && !img_data->wait_completion_event )
 		{
-			if( destroy_xshm_segment( img_data->segment->shmid ) )
+			if( destroy_xshm_segment( img_data->segment->shmseg ) )
 				return ;
 			img_data->segment = NULL ;
 		}
@@ -1406,7 +1410,7 @@ ximage2shmseg( XImage *xim )
 void registerXShmImage( XImage *ximage, XShmSegmentInfo* shminfo )
 {
 	ASXShmImage *data = safecalloc( 1, sizeof(ASXShmImage));
-	LOCAL_DEBUG_OUT( "XSHMIMAGE> CREATE_XIM : image created: xiom = %p, shminfo = %p, segment = %d, data = %p", ximage, shminfo, shminfo->shmid, ximage->data );
+	LOCAL_DEBUG_OUT( "XSHMIMAGE> CREATE_XIM : img_data = %p : image created: xiom = %p, shminfo = %p, segment = %d, data = %p", data, ximage, shminfo, shminfo->shmid, ximage->data );
 	data->ximage = ximage ;
 	data->segment = shminfo ;
 	data->size = ximage->bytes_per_line * ximage->height ;
