@@ -121,6 +121,7 @@ TermDef       FuncTerms[F_FUNCTIONS_NUM + 1] = {
 	FUNC_TERM ("MiniPixmap", 10, F_MINIPIXMAP),	/* MiniPixmap "name" */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Exec", 4, F_EXEC),	/* Exec   "name" command */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Module", 6, F_MODULE),	/* Module "name" command */
+    FUNC_TERM2 (NEED_NAME | NEED_CMD, "ExecInTerm", 10, F_ExecInTerm),   /* ExecInTerm   "name" command */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "KillModuleByName", 16, F_KILLMODULEBYNAME),	/* KillModuleByName "name" module */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "QuickRestart", 12, F_QUICKRESTART),	/* QuickRestart "name" what */
 	FUNC_TERM2 (NEED_NAME | NEED_CMD, "Background", 10, F_CHANGE_BACKGROUND),	/* Background "name" file_name */
@@ -172,6 +173,7 @@ TermDef       FuncTerms[F_FUNCTIONS_NUM + 1] = {
     FUNC_TERM ("PinMenu", 7, F_PIN_MENU),    /* PinMenu ["name"] */
 	FUNC_TERM ("TakeWindowShot", 14, F_TAKE_WINDOWSHOT),
 	FUNC_TERM ("TakeFrameShot", 13, F_TAKE_FRAMESHOT),
+	FUNC_TERM ("SwallowWindow", 13, F_SWALLOW_WINDOW),  /* SwallowWindow "name" module_name */ 
 	/* end of window functions */
 	/* these are commands  to be used only by modules */
 	FUNC_TERM ("&nonsense&", 10, F_MODULE_FUNC_START),	/* not really a command */
@@ -189,7 +191,6 @@ TermDef       FuncTerms[F_FUNCTIONS_NUM + 1] = {
     {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD, "MaxSwallow", 10, TT_FUNCTION, F_MaxSwallow, NULL},
     {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD, "SwallowModule", 13, TT_FUNCTION, F_SwallowModule, NULL},
     {TF_NO_MYNAME_PREPENDING | NEED_NAME | NEED_CMD, "MaxSwallowModule", 16, TT_FUNCTION, F_MaxSwallowModule, NULL},
-    FUNC_TERM2 (NEED_NAME | NEED_CMD, "DropExec", 8, F_DropExec),   /* DropExec   "name" command */
     {TF_NO_MYNAME_PREPENDING, "Size", 4, TT_FUNCTION, F_Size, NULL},
     {TF_NO_MYNAME_PREPENDING, "Transient", 9, TT_FUNCTION, F_Transient, NULL},
 
@@ -262,7 +263,7 @@ SetMyName (char *argv0)
 
 /* If you change/add options please change InitMyApp below and option flags in aftersteplib.h */
 
-CommandLineOpts as_standard_cmdl_options[20] =
+CommandLineOpts as_standard_cmdl_options[STANDARD_CMDL_OPTS_NUM] =
 {
 #define  SHOW_VERSION   0
 #define  SHOW_CONFIG    1
@@ -272,7 +273,7 @@ CommandLineOpts as_standard_cmdl_options[20] =
 /* 2*/{"h", "help",    "Display uasge information and stop",   NULL, handler_show_info, NULL, SHOW_USAGE},
 /* 3*/{NULL,"debug",   "Debugging: Run in Synchronous mode",  NULL, handler_set_flag, &(as_app_args.flags), ASS_Debugging},
 /* 4*/{"s", "single",  "Run on single screen only", NULL, handler_set_flag, &(as_app_args.flags), ASS_SingleScreen},
-/* 5*/{"r", "restart", "Run as if it was restarted","same as regular startup, only runs RestartFunction\ninstead of InitFunction",
+/* 5*/{"r", "restart", "Run as if it was restarted","same as regular startup, only \nruns RestartFunctioninstead of InitFunction",
                                                            handler_set_flag, &(as_app_args.flags), ASS_Restarting},
 #define OPTION_HAS_ARGS     6
 /* 6*/{"d", "display", "Specify what X display we should connect to","Overrides $DISPLAY environment variable",
@@ -299,10 +300,16 @@ CommandLineOpts as_standard_cmdl_options[20] =
 /*16*/{NULL, "trace-func","Debugging: Trace calls to a function with requested name", NULL,
                                                            handler_set_string, &(as_app_args.trace_calls), 0, CMO_HasArgs },
 #endif
-/*17*/{"l", "log","Save all output into the file instead of printing it to console", NULL,
+/*17*/{"l", "log","Save all output into the file", "(instead of printing it to console)",
                                                            handler_set_string, &(as_app_args.log_file), 0, CMO_HasArgs },
-/*18*/{"L", "locale","Set language locale to be used while displaying text", NULL,
+/*18*/{"L", "locale","Set language locale", "to be used while displaying text",
                                                            handler_set_string, &(as_app_args.locale), 0, CMO_HasArgs },
+/*19*/{NULL, "myname","Overrides module name", "will be used while parsing config files\nand reporting to AfterStep",
+                                                           handler_set_string, &(MyName), 0, CMO_HasArgs },
+/*20*/{NULL, "geometry","Overrides module's geometry", NULL,
+                                                           handler_set_geometry, &(as_app_args.geometry), 0, CMO_HasArgs },
+/*21*/{NULL, "gravity","Overrides module's gravity", NULL,
+                                                           handler_set_gravity, &(as_app_args.gravity), 0, CMO_HasArgs },
       {NULL, NULL, NULL, NULL, NULL, NULL, 0 }
 };
 
@@ -414,6 +421,34 @@ void  handler_set_int( char *argv, void *trg, long param )
         *i = atoi(argv);
 }
 
+void  handler_set_geometry( char *argv, void *trg, long param )
+{
+    register ASGeometry *geom = trg ;
+    if( argv )
+	{
+		memset( geom, 0x00, sizeof(ASGeometry));
+		parse_geometry (argv, &(geom->x), &(geom->y), &(geom->width), &(geom->height), &(geom->flags) );
+	}
+}
+
+void  handler_set_gravity( char *argv, void *trg, long param )
+{
+    register int *i = trg ;
+    *i = ForgetGravity ;
+	if( argv )
+	{
+
+		if( mystrncasecmp( argv, "NorthWest", 9 )== 0 )	  *i = NorthWestGravity ; 
+		else if( mystrncasecmp( argv, "SouthWest", 9 )== 0 )	  *i = SouthWestGravity ; 
+		else if( mystrncasecmp( argv, "NorthEasth", 10 )== 0 )	*i = NorthEastGravity ; 
+		else if( mystrncasecmp( argv, "SouthEast", 9 )== 0 )	  *i = SouthEastGravity ; 
+		else if( mystrncasecmp( argv, "Center", 6 )== 0 )	  *i = CenterGravity ; 
+		else if( mystrncasecmp( argv, "Static", 6 )== 0 )	  *i = StaticGravity ; 
+		else show_warning( "unknown gravity type \"%s\". Use one of: NorthWest, SouthWest, NorthEast, SouthEast, Center, Static", argv );
+	}
+}
+
+
 int
 match_command_line_opt( char *argvi, CommandLineOpts *options )
 {
@@ -563,8 +598,14 @@ InitMyApp (  const char *app_class, int argc, char **argv, void (*version_func)
     trace_enable_function(as_app_args.trace_calls);
 #endif
 
-	asxml_var_insert("minipixmap.width", 48);
-	asxml_var_insert("minipixmap.height", 48);
+	asxml_var_insert("icon.button.width", 64);
+	asxml_var_insert("icon.button.height", 64);
+	asxml_var_insert("icon.width", 48);
+	asxml_var_insert("icon.height", 48);
+	asxml_var_insert("minipixmap.width", 24);
+	asxml_var_insert("minipixmap.height", 24);
+	asxml_var_insert("title.font.size", 14);
+	asxml_var_insert("menu.font.size", 16);
 
 }
 
@@ -673,13 +714,16 @@ void
 InitSession()
 {
     /* initializing our dirs names */
-    Session = GetNCASSession(ASDefaultScr, as_app_args.override_home, as_app_args.override_share);
-    if( as_app_args.override_config )
-        set_session_override( Session, as_app_args.override_config, 0 );
-    if( as_app_args.override_look )
-        set_session_override( Session, as_app_args.override_look, F_CHANGE_LOOK );
-    if( as_app_args.override_feel )
-        set_session_override( Session, as_app_args.override_feel, F_CHANGE_FEEL );
+	if( Session == NULL ) 
+	{	
+    	Session = GetNCASSession(ASDefaultScr, as_app_args.override_home, as_app_args.override_share);
+    	if( as_app_args.override_config )
+        	set_session_override( Session, as_app_args.override_config, 0 );
+    	if( as_app_args.override_look )
+        	set_session_override( Session, as_app_args.override_look, F_CHANGE_LOOK );
+    	if( as_app_args.override_feel )
+        	set_session_override( Session, as_app_args.override_feel, F_CHANGE_FEEL );
+	}
 }
 
 
@@ -710,6 +754,7 @@ FreeMyAppResources()
     flush_asbidirlist_memory_pool();
     free( as_app_args.saved_argv );
     destroy_assession( Session );
+	Session = NULL ;
 	destroy_asenvironment( &Environment );
 	is_executable_in_path ( NULL );
 #ifdef XSHMIMAGE
