@@ -300,6 +300,7 @@ read_wm_hints (ASRawHints * hints, Window w)
 			XFree (hints->wm_hints);
 
 		if ((hints->wm_hints = XGetWMHints (dpy, w)) != NULL)
+		{	
 			if (get_flags (hints->wm_hints->flags, WindowGroupHint) && hints->wm_hints->window_group != w)
 			{
 				ASParentHints parent_hints;
@@ -307,10 +308,37 @@ read_wm_hints (ASRawHints * hints, Window w)
 				if (parent_hints_func (hints->wm_hints->window_group, &parent_hints))
 				{
 					if (hints->group_leader == NULL)
-						hints->group_leader = (ASParentHints *) safemalloc (sizeof (ASParentHints));
+						hints->group_leader = (ASParentHints *) safecalloc (1, sizeof (ASParentHints));
 					*(hints->group_leader) = parent_hints;
 				}
 			}
+			if( get_flags (hints->wm_hints->flags, IconWindowHint) )
+			{
+				/* some apps are written by truly insane ppl, most notably those 
+				 * designed to be docked in Window Maker's dock 
+				 * So lets bite the bullet and check if it is indeed sane :*/
+				if( hints->wm_hints->icon_window != w )
+				{
+					unsigned int width, height ;
+					if( !validate_drawable (hints->wm_hints->icon_window, &width, &height) )
+						hints->wm_hints->icon_window = None ;
+					else if( get_parent_window( hints->wm_hints->icon_window ) == w ) 
+					{	
+						if( width == 1 && height == 1 ) 
+						{      /* broken wmdock app - icon is animated, yet not properly sized */
+							XMoveResizeWindow( dpy, hints->wm_hints->icon_window, 0, 0, 64, 64 );
+						}
+						set_flags( hints->wm_hints->flags, IconWindowIsChildHint );
+						hints->wm_hints->icon_window = w ;
+					}
+				}
+				/* while merging hints we will disable iconification for the window that 
+				 * has icon window same as client */
+
+				if( hints->wm_hints->icon_window == None )
+					clear_flags (hints->wm_hints->flags, IconWindowHint);
+			}	 
+		}
 	}
 }
 
@@ -344,7 +372,7 @@ read_wm_transient_for (ASRawHints * hints, Window w)
 			if (transient_for != w && parent_hints_func (transient_for, &parent_hints))
 			{
 				if (hints->transient_for == NULL)
-					hints->transient_for = (ASParentHints *) safemalloc (sizeof (ASParentHints));
+					hints->transient_for = (ASParentHints *) safecalloc (1, sizeof (ASParentHints));
 				*(hints->transient_for) = parent_hints;
 			}
 	}
@@ -550,6 +578,16 @@ read_extwm_desktop (ASRawHints * hints, Window w)
 	}
 }
 
+CARD32
+read_extwm_desktop_val ( Window w)
+{
+	CARD32 val = 0;
+	if ( w != None)
+		read_32bit_property (w, _XA_NET_WM_DESKTOP, &val);
+	return val;
+}
+
+
 void
 read_extwm_window_type (ASRawHints * hints, Window w)
 {
@@ -1054,9 +1092,10 @@ handle_client_property_update (Window w, Atom property, ASRawHints * raw)
 			read_func = read_extwm_state;
 		else
 			return False;
-
+		
 		read_func (raw, w);
-		return True;
+		
+		return (validate_drawable( w, NULL, NULL )==w);
 	}
 	return False;
 }
@@ -1081,7 +1120,7 @@ handle_manager_property_update (Window w, Atom property, ASRawHints * raw)
 			read_extwm_icon_name (raw, w);
 			read_extwm_visible_name (raw, w);
 			read_extwm_visible_icon_name (raw, w);
-			return True;
+			return (validate_drawable( w, NULL, NULL )==w);
 		} else if (property == _XA_WM_HINTS)
 			read_func = read_wm_hints;
 		else if (property == _XA_WM_NORMAL_HINTS)
@@ -1102,13 +1141,31 @@ handle_manager_property_update (Window w, Atom property, ASRawHints * raw)
 			return False;
 		}
 		read_func (raw, w);
-		return True;
+		return (validate_drawable( w, NULL, NULL )==w);
 	} else
 		show_debug (__FILE__, __FUNCTION__, __LINE__, "incomplete parameters (%X, %X, %p)", w, property, raw);
 
 	return False;
 }
 
+Bool 
+get_extwm_state_flags (Window w, ASFlagType *flags)
+{
+	if (flags && w != None)
+	{
+		CARD32         *protocols;
+		long          nprotos = 0;
+
+        if (read_32bit_proplist (w, _XA_NET_WM_STATE, 6, &protocols, &nprotos))
+		{
+			translate_atom_list (flags, EXTWM_State, protocols, nprotos);
+			free (protocols);
+			return True;
+		}
+	}
+	return False;
+}
+
 
 
 /**********************************************************************************/
@@ -1145,6 +1202,11 @@ set_client_state (Window w, struct ASStatusHints *status)
 				extwm_states[used++] = _XA_NET_WM_STATE_SHADED;
 				gnome_state |= WIN_STATE_SHADED;
 			}
+			if (get_flags (status->flags, AS_Hidden))
+			{
+				extwm_states[used++] = _XA_NET_WM_STATE_HIDDEN;
+				gnome_state |= WIN_STATE_HIDDEN;
+			}
 			if (get_flags (status->flags, AS_MaximizedX))
 			{
 				extwm_states[used++] = _XA_NET_WM_STATE_MAXIMIZED_HORZ;
