@@ -28,6 +28,7 @@ typedef enum
     HINTS_Supported
 }HintsTypes;
 
+#if 0
 /***********************************************************/
 /* AS Startup flags :                                      */
 #define AS_StartPosition        (1<<0)
@@ -121,7 +122,7 @@ typedef enum
 #define ASHINTS_STATIC_DATA 	 28     /* number of elements below that are not */
                                         /* dynamic arrays */
 
-
+#endif
 /************************************************************************/
 /*		globals (atom IDs)					*/
 /************************************************************************/
@@ -177,6 +178,8 @@ extern Atom _XA_NET_WM_STATE_BELOW;
 extern Atom _XA_NET_WM_PID;
 extern Atom _XA_NET_WM_PING;
 
+extern Atom _XA_AS_MOVERESIZE;
+
 extern struct AtomXref      *EXTWM_State;
 
 #define IsNameProp(a) \
@@ -209,6 +212,9 @@ extern struct AtomXref      *EXTWM_State;
 /* 		ICCCM window hints		 			*/
 /************************************************************************/
 
+/* our own enchancement to XWMHints flags : */
+#define IconWindowIsChildHint	(1L << 14)
+
 
 /*
 All of this stuff is part of standard Xlib so we put it into comments -
@@ -576,6 +582,9 @@ ASRawHints;
 /*                           Interface                                   */
 /*************************************************************************/
 /* low level */
+CARD32 read_extwm_desktop_val ( Window w);
+
+
 /* X Resources : */
 void init_xrm();
 void load_user_database();
@@ -588,6 +597,8 @@ ASRawHints *collect_hints (struct ScreenInfo *scr, Window w, ASFlagType what, AS
 void destroy_raw_hints( ASRawHints *hints, Bool reusable );
 void clientprops_cleanup ();
 
+void read_wm_protocols (ASRawHints * hints, Window w);
+
 /* printing functions :
  * if func and stream are not specified - fprintf(stderr) is used ! */
 void print_wm_hints         ( stream_func func, void* stream, XWMHints *hints );
@@ -602,6 +613,9 @@ void print_hints            ( stream_func func, void* stream, ASRawHints *hints
 Bool handle_client_property_update ( Window w, Atom property, ASRawHints *raw );
 Bool handle_manager_property_update( Window w, Atom property, ASRawHints *raw );
 
+Bool get_extwm_state_flags (Window w, ASFlagType *flags);
+
+
 /*************************************************************************/
 /****************** Setting properties - updating hints : ****************/
 /*************************************************************************/
