@@ -76,6 +76,19 @@ Atom          _AS_CURRENT_DESK = None;
 Atom          _AS_CURRENT_VIEWPORT = None;
 Atom  		  _AS_SERVICE_WINDOW = None;
 
+Atom  		_AS_TBAR_PROPS		        = None;
+Atom  		_AS_BUTTON_CLOSE	     	= None;
+Atom  		_AS_BUTTON_CLOSE_PRESSED 	= None;
+Atom  		_AS_BUTTON_MAXIMIZE	     	= None;
+Atom  		_AS_BUTTON_MAXIMIZE_PRESSED = None;
+Atom  		_AS_BUTTON_MINIMIZE	     	= None;
+Atom  		_AS_BUTTON_MINIMIZE_PRESSED = None;
+Atom  		_AS_BUTTON_SHADE 			= None;
+Atom  		_AS_BUTTON_SHADE_PRESSED	= None;
+Atom  		_AS_BUTTON_MENU 			= None;
+Atom  		_AS_BUTTON_MENU_PRESSED 	= None;
+
+
 /* Crossreferences of atoms into flag value for
    different atom list type of properties :*/
 
@@ -124,7 +137,22 @@ AtomXref  _WMPropAtoms[] = {
     {"_AS_CURRENT_DESK", &_AS_CURRENT_DESK},   /* current afterstep desk */
     {"_AS_CURRENT_VIEWPORT", &_AS_CURRENT_VIEWPORT},   /* current afterstep viewport */
     {"_AS_SERVICE_WINDOW", &_AS_SERVICE_WINDOW},   /* current afterstep Scr.ServiceWin */
-    {NULL, NULL, 0, None}
+    
+#define WMPROPS_ATOM_DESC(a)   { #a, &a }
+
+    WMPROPS_ATOM_DESC(_AS_TBAR_PROPS),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_CLOSE),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_CLOSE_PRESSED),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MAXIMIZE),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MAXIMIZE_PRESSED),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MINIMIZE),
+	WMPROPS_ATOM_DESC(_AS_BUTTON_MINIMIZE_PRESSED),
+	WMPROPS_ATOM_DESC(_AS_BUTTON_SHADE),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_SHADE_PRESSED),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MENU),
+    WMPROPS_ATOM_DESC(_AS_BUTTON_MENU_PRESSED),
+	
+	{NULL, NULL, 0, None}
 };
 
 AtomXref  *WMPropAtoms = &_WMPropAtoms[0];
@@ -616,6 +644,27 @@ read_as_service_window (ASWMProps * wmprops, Bool deleted)
 	return False;
 }
 
+Bool
+read_as_tbar_props (ASWMProps * wmprops, Bool deleted)
+{
+	Bool          res = False;
+
+	if (wmprops)
+	{
+		if (wmprops->as_tbar_props_data)
+		{
+			free (wmprops->as_tbar_props_data);
+			wmprops->as_tbar_props_data = NULL;
+		}
+		wmprops->as_tbar_props_size = 0;
+		if (deleted)
+			return False;
+		res = read_as_property (wmprops->selection_window, _AS_TBAR_PROPS,
+								&(wmprops->as_tbar_props_size), &(wmprops->as_tbar_props_version), &(wmprops->as_tbar_props_data));
+	}
+	return res;
+}
+
 
 
 /******************** Property management functions: **************************/
@@ -677,6 +726,8 @@ prop_description_struct WMPropsDescriptions_volitile[] = {
 	{&_AS_MODULE_SOCKET, read_as_module_socket, WMC_ASModule, 0},
 	{&_AS_VIRTUAL_ROOT, read_as_virtual_root, WMC_ASVirtualRoot, 0},
 	{&_AS_SERVICE_WINDOW, read_as_service_window, WMC_ASServiceWindow, 0},
+	{&_AS_TBAR_PROPS, read_as_tbar_props, WMC_ASTBarProps, 0},
+	
 	{NULL, NULL, 0, 0}
 };
 
@@ -965,6 +1016,35 @@ set_as_background (ASWMProps * wmprops, Pixmap new_pmap)
 	}
 }
 
+void
+set_xrootpmap_id (ASWMProps * wmprops, Pixmap new_pmap)
+{
+	if (wmprops)
+	{
+        set_32bit_property (wmprops->scr->Root, _XROOTPMAP_ID, XA_PIXMAP, new_pmap);
+		XFlush (dpy);
+        wmprops->root_pixmap = new_pmap;
+	}
+}
+
+void
+validate_rootpmap_props(ASWMProps *wmprops)
+{
+	if( wmprops )
+	{
+		if( wmprops->root_pixmap )
+		{	
+			if( !validate_drawable( wmprops->root_pixmap, NULL, NULL ) )
+	    		set_xrootpmap_id (wmprops, None );
+		}
+		if( wmprops->as_root_pixmap ) 
+		{	
+			if( !validate_drawable( wmprops->as_root_pixmap, NULL, NULL ) )
+	    		set_as_background (wmprops, None );
+		}
+	}
+}
+
 
 CARD32
 as_desk2ext_desk (ASWMProps * wmprops, INT32 as_desk)
@@ -1184,16 +1264,6 @@ flush_wmprop_data (ASWMProps * wmprops, ASFlagType what)
 	}
 }
 
-void
-set_xrootpmap_id (ASWMProps * wmprops, Pixmap new_pmap)
-{
-	if (wmprops)
-	{
-        set_32bit_property (wmprops->scr->Root, _XROOTPMAP_ID, XA_PIXMAP, new_pmap);
-		XFlush (dpy);
-        wmprops->root_pixmap = new_pmap;
-	}
-}
 
 void
 set_clients_list (ASWMProps * wmprops, Window *list, int nclients)
@@ -1246,6 +1316,94 @@ set_active_window_prop (ASWMProps * wmprops, Window active)
 	}
 }
 
+ASTBarProps *
+get_astbar_props( ASWMProps *wmprops ) 
+{
+	ASTBarProps *tbar_props = NULL;
+	if( wmprops && wmprops->as_tbar_props_size > 0 && wmprops->as_tbar_props_data != NULL ) 
+	{
+		CARD32 *prop = wmprops->as_tbar_props_data ;
+		int i, nbuttons ; 
+		tbar_props = safecalloc( 1, sizeof(ASTBarProps));
+		tbar_props->align = prop[0] ; 
+		tbar_props->bevel = prop[1] ; 
+		tbar_props->title_h_spacing = prop[2] ;
+		tbar_props->title_v_spacing = prop[3] ;
+		tbar_props->buttons_h_border = prop[4] ; 
+		tbar_props->buttons_v_border = prop[5] ; 
+		tbar_props->buttons_spacing = prop[6] ; 
+		tbar_props->buttons_num = prop[7] ; 
+		nbuttons = tbar_props->buttons_num;
+		if( nbuttons > 10 || (nbuttons*4+8)*sizeof(CARD32) > wmprops->as_tbar_props_size)
+		{
+			tbar_props->buttons_num = 0 ;
+		}else
+		{		 
+			tbar_props->buttons = safemalloc( nbuttons*sizeof(struct ASButtonPropElem));
+			for( i = 0 ; i < nbuttons ; ++i ) 
+			{
+				tbar_props->buttons[i].kind = prop[8+i*3];
+				tbar_props->buttons[i].pmap = prop[8+i*3+1];
+				tbar_props->buttons[i].mask = prop[8+i*3+2];
+				tbar_props->buttons[i].alpha = prop[8+i*3+3];
+			}	 
+		}
+	}	 
+	
+	return tbar_props;	   
+	
+}
+
+void
+set_astbar_props( ASWMProps *wmprops, ASTBarProps *tbar_props ) 
+{
+	CARD32 *prop = NULL;
+	int size = 0;
+	CARD32 version = (1 << 8) + 1 ; /* version 1.1 */
+
+	if (wmprops == NULL || wmprops->selection_window == None )
+			return;
+
+	if( tbar_props ) 
+	{
+		int i, nbuttons = tbar_props->buttons_num;
+		size = sizeof (CARD32) * (nbuttons*4 + 8);
+		prop = safecalloc ( 1, size );
+		prop[0] = tbar_props->align ; 
+		prop[1] = tbar_props->bevel ; 
+		prop[2] = tbar_props->title_h_spacing ;
+		prop[3] = tbar_props->title_v_spacing ;
+		prop[4] = tbar_props->buttons_h_border ; 
+		prop[5] = tbar_props->buttons_v_border ; 
+		prop[6] = tbar_props->buttons_spacing ; 
+		prop[7] = tbar_props->buttons_num ; 
+	
+
+		for( i = 0 ; i < nbuttons ; ++i ) 
+		{
+			prop[8+i*3] = tbar_props->buttons[i].kind ;
+			prop[8+i*3+1] = tbar_props->buttons[i].pmap ;
+			prop[8+i*3+2] = tbar_props->buttons[i].mask ;
+			prop[8+i*3+3] = tbar_props->buttons[i].alpha ;
+		}	 
+	}
+
+	if (prop == NULL || size == 0)
+		XDeleteProperty (dpy, wmprops->selection_window, _AS_TBAR_PROPS);
+	else
+		set_as_property (wmprops->selection_window, _AS_TBAR_PROPS, prop, size, version);
+
+	if (wmprops->as_tbar_props_data )
+		free (wmprops->as_tbar_props_data);
+	
+	wmprops->as_tbar_props_data  = prop ;
+
+	wmprops->as_tbar_props_size = size;
+	wmprops->as_tbar_props_version = version;
+}
+
+
+
 /********************************************************************************/
 /* We need to handle Property Notify Events : 						            */
 /********************************************************************************/
