@@ -108,7 +108,7 @@ typedef struct ASWharfButton
     int             folder_x, folder_y;
     unsigned int    folder_width, folder_height;
 
-    FunctionData   *fdata;
+    FunctionData   *fdata[Button5];
 
     struct ASWharfFolder   *folder;
     struct ASWharfFolder   *parent;
@@ -171,10 +171,16 @@ typedef struct ASWharfState
 
 	ASWharfButton *focused_button ;
 
+	FunctionData   *default_action[Button5];
+
 }ASWharfState;
 
 ASWharfState WharfState;
 WharfConfig *Config = NULL;
+int Rows_override = -1 ;
+int Columns_override = -1 ;
+
+Atom _AS_WHARF_CLOSE = None ;
 
 #define WHARF_BUTTON_EVENT_MASK   (ButtonReleaseMask |\
                                    ButtonPressMask | LeaveWindowMask | EnterWindowMask |\
@@ -194,10 +200,11 @@ ASWharfFolder *build_wharf_folder( WharfButton *list, ASWharfButton *parent, Boo
 Bool display_wharf_folder( ASWharfFolder *aswf, int left, int top, int right, int bottom );
 Bool display_main_folder();
 void withdraw_wharf_folder( ASWharfFolder *aswf );
+static inline void withdraw_wharf_subfolders( ASWharfFolder *aswf );
 void on_wharf_moveresize( ASEvent *event );
 void destroy_wharf_folder( ASWharfFolder **paswf );
 void on_wharf_pressed( ASEvent *event );
-void release_pressure();
+void release_pressure( int button );
 Bool check_pending_swallow( ASWharfFolder *aswf );
 void exec_pending_swallow( ASWharfFolder *aswf );
 void check_swallow_window( ASWindowData *wd );
@@ -219,14 +226,27 @@ void DeadPipe(int);
 int
 main (int argc, char **argv)
 {
-    /* Save our program name - for error messages */
+    int i;
+	/* Save our program name - for error messages */
 	set_DeadPipe_handler(DeadPipe);
     InitMyApp (CLASS_WHARF, argc, argv, NULL, NULL, 0 );
 	LinkAfterStepConfig();
+    for( i = 1 ; i< argc ; ++i)
+	{
+		LOCAL_DEBUG_OUT( "argv[%d] = \"%s\", original argv[%d] = \"%s\"", i, argv[i], i, MyArgs.saved_argv[i]);	  
+		if( argv[i] != NULL )
+		{ 	
+	    	if( strcmp( argv[i] , "--rows" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
+				Rows_override = atoi( argv[i+1] );
+	    	else if( strcmp( argv[i] , "--cols" ) == 0 && i+1 < argc &&  argv[i+1] != NULL )
+				Columns_override = atoi( argv[i+1] );
+		}
+	}
 
     memset( &WharfState, 0x00, sizeof(WharfState));
 
     ConnectX( ASDefaultScr, EnterWindowMask );
+	_AS_WHARF_CLOSE = XInternAtom( dpy, "_AS_WHARF_CLOSE", False );
     ConnectAfterStep (M_TOGGLE_PAGING |
                     M_NEW_DESKVIEWPORT |
                     M_END_WINDOWLIST |
@@ -342,10 +362,18 @@ CheckConfigSanity()
     if( Config == NULL )
         Config = CreateWharfConfig ();
 
+	if( MyArgs.geometry.flags != 0 ) 
+		Config->geometry = MyArgs.geometry ;
+
+	if( Rows_override >= 0 ) 
+		Config->rows = Rows_override ;
+	if( Columns_override >= 0 ) 
+		Config->columns = Columns_override ;
+
     if( Config->rows <= 0 && Config->columns <= 0 )
         Config->rows = 1;
-
-    mystyle_get_property (Scr.wmprops);
+    
+	mystyle_get_property (Scr.wmprops);
 
     sprintf( buf, "*%sTile", get_application_name() );
     LOCAL_DEBUG_OUT("Attempting to use style \"%s\"", buf);
@@ -512,7 +540,7 @@ GetBaseOptions (const char *filename)
 {
     START_TIME(started);
 
-	ReloadASEnvironment( NULL, NULL, NULL, False );
+	ReloadASEnvironment( NULL, NULL, NULL, False, True );
 
     SHOW_TIME("BaseConfigParsingTime",started);
 }
@@ -641,7 +669,7 @@ DispatchEvent (ASEvent * event)
             on_wharf_pressed( event );
             break;
         case ButtonRelease:
-            release_pressure();
+            release_pressure(event->x.xbutton.button);
 			{
 			    ASMagic *obj = fetch_object( event->w ) ;
 				if( obj && obj->magic == MAGIC_WHARF_BUTTON )
@@ -700,11 +728,16 @@ DispatchEvent (ASEvent * event)
                 LOCAL_DEBUG_OUT("ClientMessage(\"%s\",data=(%lX,%lX,%lX,%lX,%lX)", name, event->x.xclient.data.l[0], event->x.xclient.data.l[1], event->x.xclient.data.l[2], event->x.xclient.data.l[3], event->x.xclient.data.l[4]);
                 XFree( name );
 #endif
-                if ( event->x.xclient.format == 32 &&
-                    event->x.xclient.data.l[0] == _XA_WM_DELETE_WINDOW )
-                {
-                    DeadPipe(0);
-                }
+                if ( event->x.xclient.format == 32 )
+				{	
+					if( event->x.xclient.data.l[0] == _XA_WM_DELETE_WINDOW )
+                	{
+                    	DeadPipe(0);
+                	}else if ( event->x.xclient.data.l[0] == _AS_WHARF_CLOSE )
+                	{
+						withdraw_wharf_subfolders( WharfState.root_folder );
+					}
+				}		  
             }
             break;
         case ReparentNotify :
@@ -737,6 +770,13 @@ DispatchEvent (ASEvent * event)
 				update_wharf_folder_styles( WharfState.root_folder, True );
 			}
             break;
+		default:
+#ifdef XSHMIMAGE
+			LOCAL_DEBUG_OUT( "XSHMIMAGE> EVENT : completion_type = %d, event->type = %d ", Scr.ShmCompletionEventType, event->x.type );
+			if( event->x.type == Scr.ShmCompletionEventType )
+				handle_ShmCompletion( event );
+#endif /* SHAPE */
+			break;
     }
 }
 
@@ -942,11 +982,14 @@ LOCAL_DEBUG_OUT( "contents %d has function %p with func = %ld", i, function, fun
 	        	if( function )
 				{
 					int func = function->func ;
-					if( IsSwallowFunc(func) || IsExecFunc(func) )
+					if( func != F_ExecInTerm && (IsSwallowFunc(func) || IsExecFunc(func)) )
 					{
 			   			disabled = (!is_executable_in_path (function->text));
 						if( disabled )
+						{
+							wb->contents[i].unavailable = True ;
 							show_warning( "Application \"%s\" cannot be found in the PATH.", function->text );
+						}
 					}else
 						disabled = False ;
 				}
@@ -986,8 +1029,6 @@ LOCAL_DEBUG_OUT( "contents %d has function %p with func = %ld", i, function, fun
         wb = list;
         while( wb )
         {
-			FunctionData *function = NULL ;
-
 			if( get_flags( wb->set_flags, WHARF_BUTTON_DISABLED ) )
 			{
 				wb = wb->next ;
@@ -996,21 +1037,43 @@ LOCAL_DEBUG_OUT( "contents %d has function %p with func = %ld", i, function, fun
 
             aswb->name = mystrdup( wb->title );
 			if( wb->contents )
-				function = wb->contents[wb->selected_content].function ;
-
-            if( function )
-            {
-                aswb->fdata = safecalloc( 1, sizeof(FunctionData) );
-                dup_func_data( aswb->fdata, function );
-                if( IsSwallowFunc(aswb->fdata->func) )
-                {
-                    set_flags( aswb->flags, ASW_SwallowTarget );
-                    register_swallow_target( aswb->fdata->name, aswb );
-                    if( aswb->fdata->func == F_MaxSwallow ||
-                        aswb->fdata->func == F_MaxSwallowModule )
-                        set_flags( aswb->flags, ASW_MaxSwallow );
-                }
-            }
+			{
+				int i ;
+				for( i = 0 ; i < wb->contents_num ; ++i )
+					if( !wb->contents[i].unavailable )
+					{
+						FunctionData *function = wb->contents[i].function ;
+			            if( function )
+            			{
+							int btn = Button1 ;
+							if( function->name != NULL && strlen(function->name) == 1 )
+							{
+								switch( function->name[0] )
+								{
+									case 'l' : btn = Button1 ;    break ;	
+									case 'm' : btn = Button2 ;    break ;	
+									case 'r' : btn = Button3 ;    break ;	
+									case '4' : btn = Button4 ;	  break ;
+									case '5' : btn = Button5 ;	  break ;
+								}		   
+							}	  
+							btn -= Button1 ;
+							if( aswb->fdata[btn] == NULL ) 
+							{	
+                				aswb->fdata[btn] = safecalloc( 1, sizeof(FunctionData) );
+                				dup_func_data( aswb->fdata[btn], function );
+                				if( IsSwallowFunc(aswb->fdata[btn]->func) )
+                				{
+                    				set_flags( aswb->flags, ASW_SwallowTarget );
+                    				register_swallow_target( aswb->fdata[btn]->name, aswb );
+                    				if( aswb->fdata[btn]->func == F_MaxSwallow ||
+                        				aswb->fdata[btn]->func == F_MaxSwallowModule )
+                        				set_flags( aswb->flags, ASW_MaxSwallow );
+                				}
+							}
+            			}
+					}
+			}
 			/* TODO:	Transient buttons are just spacers - they should not 
 			 * 			have any balloons displayed on them , nor they should 
 			 * 			interpret any clicks
@@ -1086,13 +1149,24 @@ destroy_wharf_folder( ASWharfFolder **paswf )
         while( --i >= 0 )
         {
             ASWharfButton *aswb = &(aswf->buttons[i]);
+			int i ;
+			if( aswb->swallowed ) 
+			{	
+				send_wm_protocol_request(aswb->swallowed->current->w, _XA_WM_DELETE_WINDOW, CurrentTime);
+				ASSync(False);
+				sleep_a_millisec(200);
+				XKillClient(dpy, aswb->swallowed->current->w );
+				ASSync(False);
+			}
             if( aswb->name )
                 free( aswb->name );
             destroy_astbar(&(aswb->bar));
             if( aswb->canvas )
                 unregister_object( aswb->canvas->w );
             destroy_ascanvas(&(aswb->canvas));
-            destroy_func_data(&(aswb->fdata));
+			for( i = 0 ; i < 5 ; ++i ) 
+				if( aswb->fdata[i] )
+            		destroy_func_data(&(aswb->fdata[i]));
             destroy_wharf_folder(&(aswb->folder));
         }
         free( aswf->buttons );
@@ -1221,6 +1295,7 @@ update_wharf_folder_transprency( ASWharfFolder *aswf, Bool force )
 			ASWharfButton *aswb= &(aswf->buttons[i]);
 			if( update_astbar_transparency( aswb->bar, aswb->canvas, force ) )
 			{
+				invalidate_canvas_save( aswb->canvas );
 				render_wharf_button( aswb );
 				update_canvas_display( aswb->canvas );
 			}
@@ -1237,12 +1312,16 @@ change_button_focus(ASWharfButton *aswb, Bool focused )
 	
 	if( aswb == NULL && focused && Scr.Look.MSWindow[BACK_FOCUSED] == NULL ) 
 		return ;
-	
-	set_astbar_focused( aswb->bar, NULL, focused );			   
-	render_wharf_button( aswb );
-	update_canvas_display( aswb->canvas );
-	update_wharf_folder_shape( aswb->parent );
 
+	if( set_astbar_focused( aswb->bar, NULL, focused ) )
+	{	
+		if( !swap_save_canvas( aswb->canvas ) )	   
+		{		
+			render_wharf_button( aswb );
+		}
+		update_canvas_display( aswb->canvas );
+		update_wharf_folder_shape( aswb->parent );
+	}
 	if( focused ) 
 		WharfState.focused_button = aswb ;
 	else if( WharfState.focused_button == aswb )
@@ -1261,6 +1340,7 @@ update_wharf_folder_styles( ASWharfFolder *aswf, Bool force )
 			set_astbar_style_ptr( aswb->bar, BAR_STATE_FOCUSED, Scr.Look.MSWindow[Scr.Look.MSWindow[BACK_FOCUSED]?BACK_FOCUSED:BACK_UNFOCUSED] );
 			if( set_astbar_style_ptr( aswb->bar, BAR_STATE_UNFOCUSED, Scr.Look.MSWindow[BACK_UNFOCUSED] ))
 			{
+				invalidate_canvas_save( aswb->canvas );
 				render_wharf_button( aswb );
 				update_canvas_display( aswb->canvas );
 			}
@@ -1282,7 +1362,7 @@ map_wharf_folder( ASWharfFolder *aswf,
 	set_folder_name( aswf, False );
 
     shints.flags = USSize|PWinGravity;
-    if( get_flags( Config->set_flags, WHARF_GEOMETRY ) )
+    if( get_flags( Config->set_flags, WHARF_GEOMETRY ) || get_flags(MyArgs.geometry.flags, XValue|YValue))
         shints.flags |= USPosition ;
     else
         shints.flags |= PPosition ;
@@ -1635,6 +1715,11 @@ display_wharf_folder( ASWharfFolder *aswf, int left, int top, int right, int bot
         (get_flags( aswf->flags, ASW_Mapped ) && !get_flags( aswf->flags, ASW_Withdrawn )) )
         return False;
 
+	if( MyArgs.gravity == SouthWestGravity || MyArgs.gravity == SouthEastGravity )
+		south = True ;
+	if( MyArgs.gravity == NorthEastGravity || MyArgs.gravity == SouthEastGravity )
+		east = True ;
+
 	if( aswf != WharfState.root_folder )
 	{
   		if( get_flags( aswf->flags, ASW_Vertical ) )
@@ -1810,9 +1895,6 @@ LOCAL_DEBUG_OUT( "unmapping canvas %p at %dx%d%+d%+d", aswf->canvas, aswf->canva
 	unmap_wharf_subfolders( aswf );
 }
 
-static inline void
-withdraw_wharf_subfolders( ASWharfFolder *aswf );
-
 void
 withdraw_wharf_folder( ASWharfFolder *aswf )
 {
@@ -1850,7 +1932,22 @@ withdraw_wharf_subfolders( ASWharfFolder *aswf )
     int i = aswf->buttons_num;
     while ( --i >= 0 )
     {
-        if( aswf->buttons[i].folder &&
+        if( aswf->buttons[i].swallowed ) 
+		{
+			ASCanvas *sc = aswf->buttons[i].swallowed->current ;
+			if( sc )
+			{	
+				XClientMessageEvent ev;
+				ev.type = ClientMessage;
+				ev.window = sc->w;
+				ev.message_type = _AS_WHARF_CLOSE;
+				ev.format = 32;
+				ev.data.l[0] = _AS_WHARF_CLOSE;
+				ev.data.l[1] = CurrentTime;
+    			XSendEvent (dpy, sc->w, False, 0, (XEvent *) & ev);
+			}
+		}	 
+		if( aswf->buttons[i].folder &&
             get_flags( aswf->buttons[i].folder->flags, ASW_Mapped ) )
             withdraw_wharf_folder( aswf->buttons[i].folder );
     }
@@ -1900,10 +1997,15 @@ exec_pending_swallow( ASWharfFolder *aswf )
         while( --i >= 0 )
         {
             if( get_flags(aswf->buttons[i].flags, ASW_SwallowTarget ) &&
-                aswf->buttons[i].fdata &&
                 aswf->buttons[i].swallowed == NULL )
             {
-                SendCommand( aswf->buttons[i].fdata, 0);
+				int k ; 
+				for( k = 0 ; k < 5 ; ++k ) 
+                	if( aswf->buttons[i].fdata[k] && IsSwallowFunc(aswf->buttons[i].fdata[k]->func)) 
+					{
+                		SendCommand( aswf->buttons[i].fdata[k], 0);
+						break;
+					}
             }
             if( aswf->buttons[i].folder )
                 exec_pending_swallow( aswf->buttons[i].folder );
@@ -1912,69 +2014,60 @@ exec_pending_swallow( ASWharfFolder *aswf )
 }
 
 void
-grab_swallowed_canvas_btns( ASCanvas *canvas, Bool action, Bool withdraw )
+grab_swallowed_canvas_btn( ASCanvas *canvas, int button, int non_lock_mods )
 {
 	register int i = 0 ;
 	register unsigned int *mods = lock_mods;
 
-	LOCAL_DEBUG_OUT( "%p,%d,%d", canvas, action, withdraw );
+	LOCAL_DEBUG_OUT( "%p,%d,0x%x", canvas, button, non_lock_mods );
 
 	if( AS_ASSERT(canvas) )
 		return;
     do
     {
         /* grab button 1 if this button performs an action */
-        if( action )
-		{	
-            XGrabButton (dpy, Button1, mods[i],
-                        canvas->w,
-                        False, ButtonPressMask | ButtonReleaseMask,
-                        GrabModeAsync, GrabModeAsync, None, None);
+        XGrabButton(dpy, button, mods[i]|non_lock_mods,
+	                canvas->w,
+                    False, ButtonPressMask | ButtonReleaseMask,
+                    GrabModeAsync, GrabModeAsync, None, None);
 #if !defined(NO_DEBUG_OUTPUT)			
 			ASSync(False);
 			fprintf( stderr, "line = %d, mods = 0x%X\n", __LINE__, mods[i] );
 #endif
-		}
-        /* grab button 3 if this is the root folder */
-        if (withdraw )
-		{	
-            XGrabButton (dpy, Button3, mods[i],
-                        canvas->w,
-                        False, ButtonPressMask | ButtonReleaseMask,
-                        GrabModeAsync, GrabModeAsync, None, None);
-#if !defined(NO_DEBUG_OUTPUT)			   
-			ASSync(False);
-			fprintf( stderr, "line = %d, canvas = %p, window = %lX, i = %d, mods = 0x%X\n", __LINE__, canvas, canvas->w, i, mods[i] );
-#endif
-		}
 		if( mods[i] == 0 )
 			return;
     }while (++i < MAX_LOCK_MODS );
 
-	if( action )
-	{	
-        XGrabButton (dpy, Button1, 0,
-                    canvas->w,
-                    False, ButtonPressMask | ButtonReleaseMask,
-                    GrabModeAsync, GrabModeAsync, None, None);
+    XGrabButton(dpy, button, non_lock_mods,
+                canvas->w,
+                False, ButtonPressMask | ButtonReleaseMask,
+                GrabModeAsync, GrabModeAsync, None, None);
 #if !defined(NO_DEBUG_OUTPUT)			   		
 		ASSync(False);
 		fprintf( stderr, "line = %d, mods = 0\n", __LINE__ );
 #endif
-	}
-    /* grab button 3 if this is the root folder */
-    if (withdraw )
-	{	
-        XGrabButton (dpy, Button3, 0,
-                    canvas->w,
-                    False, ButtonPressMask | ButtonReleaseMask,
-                    GrabModeAsync, GrabModeAsync, None, None);
-#if !defined(NO_DEBUG_OUTPUT)			   			
-			ASSync(False);
-			fprintf( stderr, "line = %d, mods = 0", __LINE__ );
-#endif
-	}
+}
 
+
+void
+grab_swallowed_canvas_btns( ASCanvas *canvas, ASWharfButton *aswb, Bool withdraw_btn )
+{	
+	if( aswb->folder!=NULL )
+		grab_swallowed_canvas_btn( canvas, Button1, 0);
+
+	if( withdraw_btn && aswb->parent == WharfState.root_folder )
+	{	
+		grab_swallowed_canvas_btn( canvas, Button3, ControlMask);
+		grab_swallowed_canvas_btn( canvas, Button3, 0);
+	}else if( aswb->fdata[Button3-Button1] ) 
+		grab_swallowed_canvas_btn( canvas, Button3, 0);
+	
+	if( aswb->fdata[Button2-Button1] ) 
+		grab_swallowed_canvas_btn( canvas, Button2, 0);
+	if( aswb->fdata[Button4-Button1] ) 
+		grab_swallowed_canvas_btn( canvas, Button4, 0);
+	if( aswb->fdata[Button5-Button1] ) 
+		grab_swallowed_canvas_btn( canvas, Button5, 0);
 }
 
 void
@@ -2020,6 +2113,7 @@ check_swallow_window( ASWindowData *wd )
 	Bool withdraw_btn ;
     ASCanvas *nc ;
     int swidth, sheight ;
+	Window icon_w = None ;
 
     if( wd == NULL && !get_flags( wd->state_flags, AS_Mapped))
         return;
@@ -2076,24 +2170,45 @@ check_swallow_window( ASWindowData *wd )
     ungrab_server();
 	ASSync(False);
     sleep_a_millisec(100);
-	grab_swallowed_canvas_btns( nc, (aswb->folder!=NULL), withdraw_btn && aswb->parent == WharfState.root_folder);
+	grab_swallowed_canvas_btns( nc, aswb, withdraw_btn );
 
-    if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) &&
-		wd->icon != None )
+    if( get_flags( wd->flags, AS_ClientIcon ) && !get_flags( wd->flags, AS_ClientIconPixmap) )
+		icon_w = wd->icon ;
+	else if( get_flags( wd->flags, AS_WMDockApp ) ) 
+	{
+		XWMHints *wm_hints ;
+		if ((wm_hints = XGetWMHints (dpy, wd->client)) != NULL)
+		{
+			if( get_flags (wm_hints->flags, IconWindowHint) )
+				icon_w = wm_hints->icon_window ;
+			XFree (wm_hints);
+		}	
+	}
+
+	if( icon_w != None ) 
     {
-        ASCanvas *ic = create_ascanvas_container( wd->icon );
+        ASCanvas *ic = create_ascanvas_container( icon_w );
         aswb->swallowed->iconic = ic;
-        XReparentWindow( dpy, wd->icon, aswb->canvas->w, (aswb->canvas->width-ic->width)/2, (aswb->canvas->height-ic->height)/2 );
-        register_object( wd->icon, (ASMagic*)aswb );
-        XSelectInput (dpy, wd->icon, StructureNotifyMask);
+        XReparentWindow( dpy, icon_w, aswb->canvas->w, (aswb->canvas->width-ic->width)/2, (aswb->canvas->height-ic->height)/2 );
+        register_object( icon_w, (ASMagic*)aswb );
+        XSelectInput (dpy, icon_w, StructureNotifyMask);
 		ASSync(False);
-        grab_swallowed_canvas_btns(  ic, (aswb->folder!=NULL), withdraw_btn && aswb->parent == WharfState.root_folder);
+		grab_swallowed_canvas_btns( ic, aswb, withdraw_btn );
     }
-    aswb->swallowed->current = ( get_flags( wd->state_flags, AS_Iconic ) &&
-                                    aswb->swallowed->iconic != NULL )?
-                                aswb->swallowed->iconic:aswb->swallowed->normal;
+    aswb->swallowed->current = aswb->swallowed->normal;
+	if( aswb->swallowed->iconic != NULL )
+	{	
+		if( get_flags( wd->flags, AS_WMDockApp ) ||
+#if 0
+			(wd->res_class && strcasecmp(wd->res_class, "DockApp") == 0) || 
+#endif				
+			get_flags( wd->state_flags, AS_Iconic ) )
+		{
+			aswb->swallowed->current = aswb->swallowed->iconic ;
+		}
+	}
     handle_canvas_config( aswb->swallowed->current );
-    LOCAL_DEBUG_OUT( "client(%lX)->icon(%lX)->current(%lX)", wd->client, wd->icon, aswb->swallowed->current->w );
+    LOCAL_DEBUG_OUT( "client(%lX)->icon(%lX)->current(%lX)", wd->client, icon_w, aswb->swallowed->current->w );
 
     if( get_flags( aswb->flags, ASW_MaxSwallow ) ||
 		(Config->force_size.width == 0 && !get_flags(aswb->flags, ASW_FixedWidth)))
@@ -2101,8 +2216,23 @@ check_swallow_window( ASWindowData *wd )
     if( get_flags( aswb->flags, ASW_MaxSwallow ) ||
 		(Config->force_size.height == 0 && !get_flags(aswb->flags, ASW_FixedHeight)) )
         aswb->desired_height = aswb->swallowed->current->height;
-    swidth = min( aswb->desired_width, aswb->swallowed->current->width );
-    sheight = min( aswb->desired_height, aswb->swallowed->current->height );
+
+    swidth = aswb->swallowed->current->width ;
+	if( get_flags( wd->flags, AS_MinSize ) && swidth < wd->hints.min_width)
+   		swidth = wd->hints.min_width;
+	if( swidth <= 2 ) 
+   		swidth = aswb->desired_width;
+	else if( swidth > aswb->desired_width ) 
+		swidth = aswb->desired_width ;
+    
+    sheight = aswb->swallowed->current->height ;
+	if( get_flags( wd->flags, AS_MinSize ) && sheight < wd->hints.min_height)
+   		sheight = wd->hints.min_height;
+	if( sheight <= 2 ) 
+   		sheight = aswb->desired_height;
+	else if( sheight > aswb->desired_height ) 
+		sheight = aswb->desired_height ;
+
     moveresize_canvas( aswb->swallowed->current,
                        make_tile_pad( get_flags(Config->align_contents,PAD_LEFT),
                                       get_flags(Config->align_contents,PAD_RIGHT),
@@ -2111,6 +2241,22 @@ check_swallow_window( ASWindowData *wd )
                                       get_flags(Config->align_contents,PAD_BOTTOM),
                                       aswb->canvas->height, sheight    ),
                        swidth, sheight );
+	if( !get_flags( wd->flags, AS_ClientIcon ) ) 
+	{  /* workaround for broken wmdock apps that draw into icon, that is a child of a client, 
+		* and at the same time not properly sized ( Just don't ask - some ppl are wierd ) */
+		XWMHints *wm_hints = XGetWMHints (dpy, wd->client);
+		LOCAL_DEBUG_OUT( "wmfire workaround step1: wm_hints = %p", wm_hints );
+		if( wm_hints ) 
+		{
+			LOCAL_DEBUG_OUT( "wmfire workaround step2: icon_window_hint = %ld, icon_window = %lX", get_flags (wm_hints->flags, IconWindowHint), wm_hints->icon_window );
+			if( get_flags (wm_hints->flags, IconWindowHint) && wm_hints->icon_window != None )				   
+			{	
+				LOCAL_DEBUG_OUT( "wmfire workaround step3: resizing to %dx%d", swidth, sheight );
+				XResizeWindow( dpy, wm_hints->icon_window, swidth, sheight );
+			}
+			XFree (wm_hints);			   
+		}
+	}
     map_canvas_window( aswb->swallowed->current, True );
     send_swallowed_configure_notify(aswb);
 	update_wharf_folder_size( aswf );
@@ -2213,8 +2359,10 @@ on_wharf_button_moveresize( ASWharfButton *aswb, ASEvent *event )
         update_astbar_transparency( aswb->bar, aswb->canvas, False );
 
     if( changes != 0 && (DoesBarNeedsRendering(aswb->bar) || is_canvas_needs_redraw( aswb->canvas )))
+	{
+		invalidate_canvas_save( aswb->canvas );
         render_wharf_button( aswb );
-
+	}
 #ifndef SHAPE
     swallowed_changes = 0 ;                    /* if no shaped extentions - ignore the changes */
 #endif
@@ -2542,13 +2690,14 @@ press_wharf_button( ASWharfButton *aswb, int state )
 }
 
 void
-release_pressure()
+release_pressure( int button )
 {
     ASWharfButton *pressed = WharfState.pressed_button ;
 LOCAL_DEBUG_OUT( "pressed button is %p", pressed );
+	button -= Button1 ;
     if( pressed )
     {
-        if( pressed->folder )
+        if( pressed->folder && !(button > 0 && pressed->fdata[button]))
         {
 LOCAL_DEBUG_OUT( "pressed button has folder %p (%s)", pressed->folder, get_flags( pressed->folder->flags, ASW_Mapped )?"Mapped":"Unmapped" );
             if( get_flags( pressed->folder->flags, ASW_Mapped ) )
@@ -2557,16 +2706,18 @@ LOCAL_DEBUG_OUT( "pressed button has folder %p (%s)", pressed->folder, get_flags
                 display_wharf_folder( pressed->folder, pressed->canvas->root_x, pressed->canvas->root_y,
                                                        pressed->canvas->root_x+pressed->canvas->width,
                                                        pressed->canvas->root_y+pressed->canvas->height  );
-        }else if( pressed->fdata )
+        }else if( pressed->fdata[button] )
         {
 #if defined(LOCAL_DEBUG) && !defined(NO_DEBUG_OUTPUT)
-            print_func_data(__FILE__, __FUNCTION__, __LINE__, pressed->fdata);
+            print_func_data(__FILE__, __FUNCTION__, __LINE__, pressed->fdata[button]);
 #endif
-            if( !get_flags( pressed->flags, ASW_SwallowTarget ) || pressed->swallowed == NULL )
+            if( button > 0 ||
+				!get_flags( pressed->flags, ASW_SwallowTarget ) || 
+				pressed->swallowed == NULL )
             {  /* send command to the AS-proper : */
 				ASWharfFolder *parentf = pressed->parent ;
 				ASWharfButton *parentb = NULL ;
-                SendCommand( pressed->fdata, 0);
+                SendCommand( pressed->fdata[button], 0);
 				while( parentf != WharfState.root_folder )
 				{
 					parentb = parentf->parent ;
@@ -2616,73 +2767,80 @@ on_wharf_pressed( ASEvent *event )
             if( (WITHDRAW_ON_EDGE(Config) && (&(aswf->buttons[0]) == aswb || &(aswf->buttons[aswf->buttons_num-1]) == aswb)) ||
                  WITHDRAW_ON_ANY(Config))
         	{
-            	if( get_flags( WharfState.root_folder->flags, ASW_Withdrawn) )
-				{
-					/* update our name to normal */	  
-					set_folder_name( WharfState.root_folder, False );
-					LOCAL_DEBUG_OUT( "un - withdrawing folder%s","" );
-                	display_main_folder();
-            	}else
-            	{
-                	int wx = 0, wy = 0, wwidth, wheight;
-                	int i = aswf->buttons_num ;
+				if( aswb->fdata[Button3-Button1] == NULL || 
+					(event->x.xbutton.state&AllModifierMask) == ControlMask )
+				{			   
+            		if( get_flags( WharfState.root_folder->flags, ASW_Withdrawn) )
+					{
+						/* update our name to normal */	  
+						set_folder_name( WharfState.root_folder, False );
+						LOCAL_DEBUG_OUT( "un - withdrawing folder%s","" );
+                		display_main_folder();
+            		}else
+            		{
+                		int wx = 0, wy = 0, wwidth, wheight;
+                		int i = aswf->buttons_num ;
                 
-					if( Config->withdraw_style < WITHDRAW_ON_ANY_BUTTON_AND_SHOW )
-                    	aswb = &(aswf->buttons[0]);
+						if( Config->withdraw_style < WITHDRAW_ON_ANY_BUTTON_AND_SHOW )
+                    		aswb = &(aswf->buttons[0]);
 
-					withdraw_wharf_subfolders( aswf );
-					/* update our name to withdrawn */	  					
-					set_folder_name( aswf, True );
+						withdraw_wharf_subfolders( aswf );
+						/* update our name to withdrawn */	  					
+						set_folder_name( aswf, True );
 
-                	wwidth = aswb->desired_width ;
-                	wheight = aswb->desired_height ;
-                	if( get_flags( Config->geometry.flags, XNegative ) )
-                    	wx = Scr.MyDisplayWidth - wwidth ;
-                	if( get_flags( Config->geometry.flags, YNegative ) )
-                    	wy = Scr.MyDisplayHeight - wheight ;
+                		wwidth = aswb->desired_width ;
+                		wheight = aswb->desired_height ;
+                		if( get_flags( Config->geometry.flags, XNegative ) )
+                    		wx = Scr.MyDisplayWidth - wwidth ;
+                		if( get_flags( Config->geometry.flags, YNegative ) )
+                    		wy = Scr.MyDisplayHeight - wheight ;
 
-					if( get_flags(Config->flags, WHARF_ANIMATE ) )
-					{	
-						set_flags(aswf->flags,ASW_UseBoundary );
-						animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, wwidth, wheight );
-						clear_flags(aswf->flags,ASW_UseBoundary );
-					}
+						if( get_flags(Config->flags, WHARF_ANIMATE ) )
+						{	
+							set_flags(aswf->flags,ASW_UseBoundary );
+							animate_wharf_loop(aswf, aswf->canvas->width, aswf->canvas->height, wwidth, wheight );
+							clear_flags(aswf->flags,ASW_UseBoundary );
+						}
 
-					LOCAL_DEBUG_OUT( "withdrawing folder to %dx%d%+d%+d", wwidth, wheight, wx, wy );
-                	XRaiseWindow( dpy, aswb->canvas->w );
-                	while ( --i >= 0 )
-                	{
-                    	if( &(aswf->buttons[i]) != aswb && 
-							aswf->buttons[i].canvas->root_x == aswf->canvas->root_x && 
-							aswf->buttons[i].canvas->root_y == aswf->canvas->root_y)
-                    	{
-                        	aswf->buttons[i].folder_x = wwidth ;
-                        	aswf->buttons[i].folder_y = wheight ;
-                        	move_canvas( aswf->buttons[i].canvas, wwidth, wheight );
-                    	}
-                	}
-                	set_flags( aswf->flags, ASW_Withdrawn );
-					set_withdrawn_clip_area( aswf, wx, wy, wwidth, wheight );
-                	moveresize_canvas( aswf->canvas, wx, wy, wwidth, wheight );
-//					ASSync(False);
-//					MapConfigureNotifyLoop();
+						LOCAL_DEBUG_OUT( "withdrawing folder to %dx%d%+d%+d", wwidth, wheight, wx, wy );
+                		XRaiseWindow( dpy, aswb->canvas->w );
+                		while ( --i >= 0 )
+                		{
+                    		if( &(aswf->buttons[i]) != aswb && 
+								aswf->buttons[i].canvas->root_x == aswf->canvas->root_x && 
+								aswf->buttons[i].canvas->root_y == aswf->canvas->root_y)
+                    		{
+                        		aswf->buttons[i].folder_x = wwidth ;
+                        		aswf->buttons[i].folder_y = wheight ;
+                        		move_canvas( aswf->buttons[i].canvas, wwidth, wheight );
+                    		}
+                		}
+                		set_flags( aswf->flags, ASW_Withdrawn );
+						set_withdrawn_clip_area( aswf, wx, wy, wwidth, wheight );
+                		moveresize_canvas( aswf->canvas, wx, wy, wwidth, wheight );
+	//					ASSync(False);
+	//					MapConfigureNotifyLoop();
 
-                	aswb->folder_x = 0;
-                	aswb->folder_y = 0;
-                	aswb->folder_width = wwidth ;
-                	aswb->folder_height = wheight ;
-                	moveresize_canvas( aswb->canvas, 0, 0, wwidth, wheight );
+                		aswb->folder_x = 0;
+                		aswb->folder_y = 0;
+                		aswb->folder_width = wwidth ;
+                		aswb->folder_height = wheight ;
+                		moveresize_canvas( aswb->canvas, 0, 0, wwidth, wheight );
 
-					aswf->withdrawn_button = aswb ;
-            	}
-            	return;
+						aswf->withdrawn_button = aswb ;
+            		}
+	            	return;
+				}
         	}
-			if( check_app_click	( aswb, &(event->x.xbutton) ) ) 
-			{
-	            event->x.xbutton.window = aswb->swallowed->current->w;
-				XSendEvent (dpy, aswb->swallowed->current->w, False, ButtonPressMask, &(event->x));	
-				return;
-			}	 
+			if( event->x.xbutton.button == Button1 || aswb->fdata[event->x.xbutton.button-Button1] == NULL ) 
+			{	
+				if( check_app_click	( aswb, &(event->x.xbutton) ) ) 
+				{
+	            	event->x.xbutton.window = aswb->swallowed->current->w;
+					XSendEvent (dpy, aswb->swallowed->current->w, False, ButtonPressMask, &(event->x));	
+					return;
+				}	 
+			}
 		}
         press_wharf_button( aswb, event->x.xbutton.state );
     }
