@@ -86,7 +86,6 @@ static int           MenuPinOnButton = -1 ;
 
 static MyStyleDefinition *MyStyleList = NULL ;
 static MyFrameDefinition *MyFrameList = NULL ;
-static char              *DefaultFrameName = NULL ;
 static MyFrameDefinition *LegacyFrameDef = NULL ;
 
 static balloonConfig BalloonConfig = {0, 0, 0, 0, 0, 0, NULL };
@@ -190,6 +189,7 @@ struct config main_config[] = {
     {"RecentSubmenuItems", SetInts, (char**)&TmpFeel.recent_submenu_items, (int*)&dummy},
 	{"WinListSortOrder", SetInts, (char**)&TmpFeel.winlist_sort_order, (int *)&dummy},
 	{"WinListHideIcons", SetFlag, (char**)WinListHideIcons, (int *)&dummy},
+	{"SuppressIcons", SetFlag, (char**)SuppressIcons, (int *)&dummy},
 
     /* look options */
 	/* obsolete stuff */
@@ -254,7 +254,7 @@ struct config main_config[] = {
     {"DeskBack"							, deskback_parse, NULL, NULL },
     {"*asetrootDeskBack"				, deskback_parse, NULL, NULL },        /* pretending to be asteroot here */
     {"MyFrame"							, myframe_parse, (char**)"afterstep", (int*)&MyFrameList},
-    {"DefaultFrame"						, assign_string, (char**)&DefaultFrameName, (int*)0},
+    {"DefaultFrame"						, assign_string, (char**)&TmpLook.DefaultFrameName, (int*)0},
     {"DontDrawBackground"				, SetLookFlag, (char **)DontDrawBackground, NULL},
     {"CursorFore"						, assign_string, &TmpLook.CursorFore, (int *)0},    /* foreground color to be used for coloring pointer's cursor */
     {"CursorBack"						, assign_string, &TmpLook.CursorBack, (int *)0},    /* background color to be used for coloring pointer's cursor */
@@ -283,6 +283,7 @@ struct config main_config[] = {
     {"StartMenuSortMode"				, SetInts, (char **)&TmpLook.StartMenuSortMode, (int *)&dummy},
     {"DrawMenuBorders"					, SetInts, (char **)&TmpLook.DrawMenuBorders, (int *)&dummy},
     {"ButtonSize"						, SetInts, (char **)&TmpLook.ButtonWidth, (int *)&TmpLook.ButtonHeight},
+    {"MinipixmapSize"					, SetInts, (char **)&TmpLook.minipixmap_width, (int *)&TmpLook.minipixmap_height},
     {"SeparateButtonTitle"				, SetLookFlag, (char **)SeparateButtonTitle, NULL},
     {"RubberBand"						, SetInts, (char **)&TmpLook.RubberBand, &dummy},
     {"DefaultStyle"						, assign_string, (char **)&MSWindowName[BACK_DEFAULT], (int *)0},
@@ -312,6 +313,9 @@ struct config main_config[] = {
 	{"CoverAnimationSteps"				, SetInts, (char**)&(TmpFeel.desk_cover_animation_steps), NULL },
 	{"CoverAnimationType"				, SetInts, (char**)&(TmpFeel.desk_cover_animation_type), NULL },
 	{"AnimateDeskChange"				, SetFlag, (char**)AnimateDeskChange, &dummy },
+	{"ButtonIconSpacing" 				, SetInts, (char **)&TmpLook.ButtonIconSpacing, &dummy},
+    {"ButtonBevel"						, bevel_parse, (char**)"afterstep", (int*)&(TmpLook.ButtonBevel)},
+    {"ButtonAlign"						, align_parse, (char**)"afterstep", (int*)&(TmpLook.ButtonAlign)},
 	{"", 0, (char **)0, (int *)0}
 };
 
@@ -498,7 +502,11 @@ init_old_look_variables (Bool free_resources)
 void
 merge_old_look_variables (MyLook *look)
 {
-    char         *button_style_names[BACK_STYLES] = {"ButtonTitleFocus", "ButtonTitleUnfocus", "ButtonTitleSticky", "ButtonTitleDefault" };
+    char         *button_style_names[BACK_STYLES] = 
+{AS_ICON_TITLE_MYSTYLE, 
+													 AS_ICON_TITLE_UNFOCUS_MYSTYLE, 
+													 AS_ICON_TITLE_STICKY_MYSTYLE, 
+													 "ButtonTitleDefault" };
     MyStyle      *button_styles[BACK_STYLES];
     int i ;
 
@@ -652,6 +660,7 @@ InitLook (MyLook *look, Bool free_resources)
 {
 	int i ;
     /* actuall MyLook cleanup : */
+
     mylook_init (look, free_resources, LL_Everything );
 
     /* other related things : */
@@ -671,8 +680,6 @@ InitLook (MyLook *look, Bool free_resources)
         unload_font (&IconFont);
         DestroyMyStyleDefinitions (&MyStyleList);
         DestroyMyFrameDefinitions (&MyFrameList);
-        if( DefaultFrameName )
-            free( DefaultFrameName );
 		if( LegacyFrameDef )
 			DestroyMyFrameDefinitions( &LegacyFrameDef );
         if( BalloonConfig.style )
@@ -702,7 +709,6 @@ InitLook (MyLook *look, Bool free_resources)
 		MSMenuName[i] = NULL ;
 
 	MyFrameList = NULL ;
-    DefaultFrameName = NULL ;
 	LegacyFrameDef = NULL ;
     memset( &BalloonConfig, 0x00, sizeof(BalloonConfig));
 }
@@ -725,8 +731,23 @@ merge_look( MyLook *to, MyLook *from )
     to->resize_move_geometry = from->resize_move_geometry ; 
     to->StartMenuSortMode = from->StartMenuSortMode ; 
     to->DrawMenuBorders = from->DrawMenuBorders ; 
-    to->ButtonWidth  = from->ButtonWidth ; 
-	to->ButtonHeight = from->ButtonHeight ; 
+    to->ButtonWidth  = (from->ButtonWidth == 0)? 64 :from->ButtonWidth  ; 
+	to->ButtonHeight = (from->ButtonHeight== 0)? to->ButtonWidth :from->ButtonHeight ; 
+	to->ButtonIconSpacing = from->ButtonIconSpacing;
+	to->ButtonAlign  = from->ButtonAlign;
+	to->ButtonBevel  = from->ButtonBevel;
+
+    to->minipixmap_width  = (from->minipixmap_width == 0)? 24 : from->minipixmap_width ; 
+	to->minipixmap_height = (from->minipixmap_height == 0)?to->minipixmap_width:from->minipixmap_height; 
+	
+	asxml_var_insert("icon.button.width", to->ButtonWidth);
+	asxml_var_insert("icon.button.height", to->ButtonHeight);
+
+	asxml_var_insert("minipixmap.width", to->minipixmap_width);
+	asxml_var_insert("minipixmap.height", to->minipixmap_height);
+
+	to->DefaultFrameName = from->DefaultFrameName ; 
+
     to->RubberBand = from->RubberBand ; 
     to->menu_icm = from->menu_icm ; 
     to->menu_hcm = from->menu_hcm ; 
@@ -780,9 +801,13 @@ make_styles (MyLook *look)
 
 MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagType default_title_align )
 {
-    MyFrame *frame = create_myframe();
+    MyFrame *frame;
     int i ;
-    frame->name = mystrdup( fd->name );
+
+	frame = get_flags(fd->flags,MYFRAME_INHERIT_DEFAULTS)? 
+				create_default_myframe(default_title_align) : create_myframe();
+    
+	frame->name = mystrdup( fd->name );
     for( i = 0 ; i < fd->inheritance_num ; ++i )
     {
 		ASHashData hdata ;
@@ -798,14 +823,14 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
             set_string_value(&(frame->part_filenames[i]), mystrdup(fd->parts[i]), NULL, 0);
         if( get_flags( fd->set_part_size, 0x01<<i ) )
         {
-            frame->part_width[i] = fd->part_width[i];
-            frame->part_length[i] = fd->part_length[i];
+            frame->part_width[i] = max(fd->part_width[i],1);
+            frame->part_length[i] = max(fd->part_length[i],1);
         }
-        if( IsPartFBevelSet(frame,i) )
+        if( IsPartFBevelSet(fd,i) )
             frame->part_fbevel[i] = fd->part_fbevel[i];
-        if( IsPartUBevelSet(frame,i) )
+        if( IsPartUBevelSet(fd,i) )
             frame->part_ubevel[i] = fd->part_ubevel[i];
-        if( IsPartSBevelSet(frame,i) )
+        if( IsPartSBevelSet(fd,i) )
             frame->part_sbevel[i] = fd->part_sbevel[i];
         if( get_flags( fd->set_part_align, 0x01<<i ) )
             frame->part_align[i] = fd->part_align[i];
@@ -872,6 +897,11 @@ MyFrame *add_myframe_from_def( ASHashTable *list, MyFrameDefinition *fd, ASFlagT
     if( get_flags( fd->set_title_attr, MYFRAME_TitleSSatSet ) )
         frame->title_ssat = fd->title_ssat;
 	
+    if( get_flags( fd->set_title_attr, MYFRAME_TitleHSpacingSet ) )
+		frame->title_h_spacing = fd->title_h_spacing;
+	if( get_flags( fd->set_title_attr, MYFRAME_TitleVSpacingSet ) )
+		frame->title_v_spacing = fd->title_v_spacing;
+
 	
 	for( i = 0 ; i < MYFRAME_TITLE_BACKS ; ++i )
 	{
@@ -989,6 +1019,8 @@ void
 FixLook( MyLook *look )
 {
     ASFlagType default_title_align = ALIGN_LEFT ;
+	int tbar_font_size = 0 ;
+	int menu_font_size = 0 ;
     int i ;
 #ifdef LOCAL_DEBUG
     LOCAL_DEBUG_OUT( "syncing %s","");
@@ -1029,6 +1061,21 @@ FixLook( MyLook *look )
     ASSync(False);
 #endif
 
+	for( i = 0 ; i < BACK_STYLES ; ++i ) 
+		if( look->MSWindow[i] )
+			if( look->MSWindow[i]->font.as_font )
+				if( look->MSWindow[i]->font.as_font->max_height > tbar_font_size ) 
+					tbar_font_size = look->MSWindow[i]->font.as_font->max_height ;
+	asxml_var_insert("title.font.size", tbar_font_size);
+	
+	for( i = 0 ; i < MENU_BACK_STYLES ; ++i ) 
+		if( look->MSMenu[i] )
+			if( look->MSMenu[i]->font.as_font )
+				if( look->MSMenu[i]->font.as_font->max_height > menu_font_size ) 
+					menu_font_size = look->MSMenu[i]->font.as_font->max_height ;
+	asxml_var_insert("menu.font.size", menu_font_size);
+
+
     mystyle_list_set_property (Scr.wmprops, look->styles_list);
 #ifdef LOCAL_DEBUG
     LOCAL_DEBUG_OUT( "syncing %s","");
@@ -1040,7 +1087,11 @@ FixLook( MyLook *look )
     else if(look->TitleTextAlign == JUSTIFY_CENTER )
         default_title_align = ALIGN_CENTER ;
 
-    /* update frame geometries */
+	if( look->DefaultFrameName == NULL ) 
+		look->DefaultFrameName = mystrdup("default");
+    check_myframes_list( look );
+    
+	/* update frame geometries */
     if (get_flags( look->flags, DecorateFrames))
     {
         MyFrameDefinition *fd ;
@@ -1050,35 +1101,31 @@ FixLook( MyLook *look )
         PrintMyFrameDefinitions (MyFrameList, 1);
 #endif
         LOCAL_DEBUG_OUT( "MyFrameList %p", MyFrameList );
-        if( MyFrameList || LegacyFrameDef )
-            check_myframes_list( look );
         for( fd = MyFrameList ; fd != NULL ; fd = fd->next )
         {
             LOCAL_DEBUG_OUT( "processing MyFrameDefinition %p", fd );
-            frame = add_myframe_from_def( look->FramesList, fd, default_title_align|ALIGN_VCENTER );
-            myframe_load ( frame, Scr.image_manager );
+            if( (frame = add_myframe_from_def( look->FramesList, fd, default_title_align|ALIGN_VCENTER )) != NULL ) 
+            	myframe_load ( frame, Scr.image_manager );
         }
-		if( LegacyFrameDef && DefaultFrameName  == NULL )
+		if( LegacyFrameDef )
 		{
             LOCAL_DEBUG_OUT( "processing legacy MyFrameDefinition %p", LegacyFrameDef );
-            frame = add_myframe_from_def( look->FramesList, LegacyFrameDef, default_title_align|ALIGN_VCENTER );
-            myframe_load ( frame, Scr.image_manager );
-			if( look->DefaultFrame == NULL )
-				look->DefaultFrame = frame ;
+			LegacyFrameDef->name = mystrdup(look->DefaultFrameName);
+            if( (frame = add_myframe_from_def( look->FramesList, LegacyFrameDef, default_title_align|ALIGN_VCENTER )) != NULL )
+            	myframe_load ( frame, Scr.image_manager );
 		}
         DestroyMyFrameDefinitions (&MyFrameList);
 		DestroyMyFrameDefinitions (&LegacyFrameDef);
-        if( look->DefaultFrame == NULL && DefaultFrameName != NULL )
-        {
-			ASHashData hdata ;
-            if( get_hash_item( look->FramesList, AS_HASHABLE(DefaultFrameName), &hdata.vptr) == ASH_Success )
-				look->DefaultFrame = hdata.vptr ;
-            LOCAL_DEBUG_OUT( "DefaultFrameName is \"%s\": found frame %p with that name.", DefaultFrameName, look->DefaultFrame );
-        }
-    }else if( look->DefaultFrame != NULL )
-		destroy_myframe( &(look->DefaultFrame) );
-    if( look->DefaultFrame == NULL )
-        look->DefaultFrame = create_default_myframe(default_title_align|ALIGN_VCENTER);
+        LOCAL_DEBUG_OUT( "DefaultFrameName is \"%s\".", look->DefaultFrameName );
+    }
+
+	if( myframe_find( look->DefaultFrameName ) == NULL ) 
+	{	
+        MyFrame *dmf = create_default_myframe(default_title_align|ALIGN_VCENTER);
+		dmf->name = mystrdup(look->DefaultFrameName);
+		if( add_hash_item( look->FramesList, AS_HASHABLE(dmf->name), dmf ) != ASH_Success ) 
+			destroy_myframe( &dmf );
+	}
 
 #ifdef LOCAL_DEBUG
     LOCAL_DEBUG_OUT( "syncing %s","");
@@ -1243,6 +1290,94 @@ redecorate_aswindow_iter_func(void *data, void *aux_data)
     return True;
 }
 
+void 
+advertise_tbar_props()
+{
+	ASTBarProps props ; 
+	MyFrame *frame = myframe_find(NULL);
+	MouseButton  *btn;
+	int i, k ; 
+
+#define TBAR_BUTTONS_OF_CONCERN 5
+	struct 
+	{
+		Atom *kind, *kind_down;
+	 	int func;
+		MyButton *btn;
+	}buttons[TBAR_BUTTONS_OF_CONCERN] = 
+	{	{ &_AS_BUTTON_CLOSE, 	&_AS_BUTTON_CLOSE_PRESSED, 		F_CLOSE, 	NULL },
+		{ &_AS_BUTTON_MAXIMIZE, &_AS_BUTTON_MAXIMIZE_PRESSED, 	F_MAXIMIZE, NULL },
+		{ &_AS_BUTTON_MINIMIZE, &_AS_BUTTON_MINIMIZE_PRESSED, 	F_ICONIFY, 	NULL },
+		{ &_AS_BUTTON_SHADE, 	&_AS_BUTTON_SHADE_PRESSED, 		F_SHADE, 	NULL },
+		{ &_AS_BUTTON_MENU, 	&_AS_BUTTON_MENU_PRESSED, 		F_POPUP, 	NULL }};
+
+	
+	props.align = frame->title_align ;
+	props.bevel = frame->title_fbevel|frame->title_ubevel ;
+	props.title_h_spacing = frame->title_h_spacing ;
+	props.title_v_spacing = frame->title_v_spacing ;
+	props.buttons_h_border = max(Scr.Look.TitleButtonXOffset[0],Scr.Look.TitleButtonXOffset[1]);
+	props.buttons_v_border = max(Scr.Look.TitleButtonYOffset[0],Scr.Look.TitleButtonYOffset[1]);
+	props.buttons_spacing = max(Scr.Look.TitleButtonSpacing[0],Scr.Look.TitleButtonSpacing[1]);
+		
+    for (btn = Scr.Feel.MouseButtonRoot; btn != NULL; btn = btn->NextButton)
+		if ( (btn->Context & C_TButtonAll) )
+		{
+			for( i = 0 ; i < TBAR_BUTTONS_OF_CONCERN ; ++i )
+			{
+				if( btn->fdata->func == buttons[i].func && buttons[i].btn == NULL )
+				{/* now lets find that button in look : */
+				 	for( k = 0  ; k < TITLE_BUTTONS ; ++k ) 
+						if ( Scr.Look.buttons[k].unpressed.image != NULL &&
+						     (btn->Context&Scr.Look.buttons[k].context) != 0 )
+						{
+							buttons[i].btn = &(Scr.Look.buttons[k]);
+							break;	
+						}
+					break;	
+				}	 
+			}
+		}
+	props.buttons_num = 0 ; 
+	for( i = 0 ; i < TBAR_BUTTONS_OF_CONCERN ; ++i )
+		if( buttons[i].btn != NULL ) 
+		{
+			++props.buttons_num	;
+			if( buttons[i].btn->pressed.image != NULL && 
+				buttons[i].btn->pressed.image != buttons[i].btn->unpressed.image )
+				++props.buttons_num	;
+		}	 
+	props.buttons = safemalloc( props.buttons_num*sizeof(struct ASButtonPropElem));
+	k = 0 ;
+	for( i = 0 ; i < TBAR_BUTTONS_OF_CONCERN ; ++i ) 
+		if( buttons[i].btn != NULL ) 
+		{
+			MyIcon *icon = &(buttons[i].btn->unpressed);
+			if( icon->pix == None ) 
+				make_icon_pixmaps( icon, False );
+	
+			props.buttons[k].kind = *(buttons[i].kind);
+			props.buttons[k].pmap = icon->pix;
+			props.buttons[k].mask = icon->mask;
+			props.buttons[k].alpha = icon->alpha;
+	   		++k ;
+			if( buttons[i].btn->pressed.image != NULL && 
+				buttons[i].btn->pressed.image != buttons[i].btn->unpressed.image )
+			{	  
+				icon = &(buttons[i].btn->pressed);
+				if( icon->pix == None ) 
+					make_icon_pixmaps( icon, False );
+				props.buttons[k].kind = *(buttons[i].kind_down);
+				props.buttons[k].pmap = icon->pix;
+				props.buttons[k].mask = icon->mask;
+				props.buttons[k].alpha = icon->alpha;
+				++k ;
+			}
+		}	 
+
+	set_astbar_props( Scr.wmprops, &props ); 
+}
+
 
 /*************************************************************************/
 /* reading confiug files now :                                           */
@@ -1339,7 +1474,7 @@ LoadASConfig (int thisdesktop, ASFlagType what)
         const char *const_configfile;
         if (get_flags(what, PARSE_BASE_CONFIG))
 		{
-			if( ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, get_flags(what, PARSE_LOOK_CONFIG) ) )
+			if( ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, get_flags(what, PARSE_LOOK_CONFIG), True ) )
 			{
 				if( !get_flags(what, PARSE_LOOK_CONFIG) )
 				{
@@ -1358,6 +1493,7 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 
         if (get_flags(what, PARSE_LOOK_CONFIG))
 		{
+
 			stop_all_background_xfer();
 			LoadColorScheme();
 
@@ -1476,7 +1612,7 @@ LoadASConfig (int thisdesktop, ASFlagType what)
         }
 	} else
 	{
-		ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, True );
+		ReloadASEnvironment( &old_image_manager, &old_font_manager, NULL, True, True );
 		LoadColorScheme();
 		memset( &TmpLook, 0x00, sizeof(TmpLook));
 		InitLook (&TmpLook, False );
@@ -1524,12 +1660,14 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 		}
     }
 
-    if( get_flags(what, PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
+    if( get_flags(what, PARSE_BASE_CONFIG|PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
 	{
+        ASHashIterator  i;
 		ARGB32 cursor_fore = ARGB32_White ;
 		ARGB32 cursor_back = ARGB32_Black ;
 
 		fix_menu_pin_on( &Scr.Look );
+		
 		/* also need to recolor cursors ! */
 		if( Scr.Look.CursorFore )
 			parse_argb_color( Scr.Look.CursorFore, &cursor_fore );
@@ -1537,23 +1675,16 @@ LoadASConfig (int thisdesktop, ASFlagType what)
 	   		parse_argb_color( Scr.Look.CursorBack, &cursor_back );
 		recolor_feel_cursors( &Scr.Feel, cursor_fore, cursor_back );
 		XDefineCursor (dpy, Scr.Root, Scr.Feel.cursors[ASCUR_Default]);
-	}
-	if( get_flags(what, PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
-	{
-        ASHashIterator  i;
-        if( start_hash_iteration (Scr.Feel.Popups, &i) )
+        
+		if( start_hash_iteration (Scr.Feel.Popups, &i) )
             do
             {
 				reload_menu_pmaps( (MenuData *)curr_hash_data(&i) );
             }while( next_hash_item( &i ));
+		
+		advertise_tbar_props();
 	}
 	   
-    /* TODO: update the menus */
-    if (get_flags(what, PARSE_BASE_CONFIG|PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG))
-	{
-		/* TODO: update mouse cursor ! */
-    }
-
     /* force update of window frames */
     if (get_flags(what, PARSE_BASE_CONFIG|PARSE_LOOK_CONFIG|PARSE_FEEL_CONFIG|PARSE_DATABASE_CONFIG))
     {
@@ -1581,8 +1712,10 @@ LoadASConfig (int thisdesktop, ASFlagType what)
     }
 
     ConfigureNotifyLoop();
+
     remove_desktop_cover();
 
+	validate_rootpmap_props(Scr.wmprops);
 }
 
 /*****************************************************************************
