@@ -118,7 +118,7 @@ ASOrientation VertOrientation =
     {7, 6, 5, 4, 3, 2, 1}
 };
 
-inline ASOrientation*
+ASOrientation*
 get_orientation_data( ASWindow *asw )
 {
     if( asw && asw->magic == MAGIC_ASWINDOW && asw->hints )
@@ -248,12 +248,12 @@ check_frame_canvas( ASWindow *asw, Bool required )
 
             /* create windows */
             valuemask = CWBorderPixel | CWCursor | CWEventMask ;
-            /*if( Scr.asv->visual_info.visual == DefaultVisual( dpy, Scr.screen ) )
-            {*/
+            if( Scr.asv->visual_info.visual == DefaultVisual( dpy, Scr.screen ) )
+            {
 			/* only if root has same depth and visual as us! */
-                attributes.background_pixmap = None;
+                attributes.background_pixmap = ParentRelative;
                 valuemask |= CWBackPixmap;
-            /*}*/
+            }
             attributes.border_pixel = Scr.asv->black_pixel;
             attributes.cursor = Scr.Feel.cursors[ASCUR_Default];
             attributes.event_mask = AS_FRAME_EVENT_MASK;
@@ -358,12 +358,12 @@ LOCAL_DEBUG_OUT( "++CREAT Client(%lx(%s))->CLIENT->canvas(%p)->window(%lx)", asw
 
 LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->CLIENT->canvas(%p)->window(%lx)", asw->w, ASWIN_NAME(asw)?ASWIN_NAME(asw):"noname", canvas, canvas->w );
         destroy_ascanvas( &canvas );
-        unregister_aswindow( w );
 
         /*
          * Prevent the receipt of an UnmapNotify in case we are simply restarting,
          * since that would cause a transition to the Withdrawn state.
 		 */
+#if 1
         if( !ASWIN_GET_FLAGS(asw, AS_Dead) && get_parent_window( w ) == asw->frame )
         {
             if( get_flags( AfterStepState, ASS_Shutdown ) )
@@ -374,9 +374,11 @@ LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->CLIENT->canvas(%p)->window(%lx)", asw
             XConfigureWindow (dpy, w, CWX|CWY|CWWidth|CWHeight|CWBorderWidth, &xwc);
         }
         XSync (dpy, 0);
+#endif
     }
 
-    return (asw->client_canvas = canvas);
+	asw->client_canvas = canvas;
+    return canvas;
 }
 
 /* creating/destroying our icon window : */
@@ -426,8 +428,9 @@ LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->ICON->canvas(%p)->window(%lx)", asw->
         else
             destroy_registered_window( w );
     }
+	asw->icon_canvas = canvas;
 
-    return (asw->icon_canvas = canvas);
+	return canvas;
 }
 
 /* creating/destroying our icon title window : */
@@ -437,16 +440,16 @@ check_icon_title_canvas( ASWindow *asw, Bool required, Bool reuse_icon_canvas )
     ASCanvas *canvas = asw->icon_title_canvas;
 	Window w;
 
-    if( canvas &&
-        ((reuse_icon_canvas && canvas != asw->icon_canvas) || !required) )
-    {
-        w = canvas->w ;
-LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->ICONT->canvas(%p)->window(%lx)", asw->w, ASWIN_NAME(asw)?ASWIN_NAME(asw):"noname", canvas, canvas->w );
-        destroy_ascanvas( &canvas );
-        destroy_registered_window( w );
-    }
-
-    if( required )
+	if( !required ) 
+	{	
+    	if( canvas && canvas != asw->icon_canvas )
+    	{
+        	w = canvas->w ;
+			LOCAL_DEBUG_OUT( "--DESTR Client(%lx(%s))->ICONT->canvas(%p)->window(%lx)", asw->w, ASWIN_NAME(asw)?ASWIN_NAME(asw):"noname", canvas, canvas->w );
+        	destroy_ascanvas( &canvas );
+        	destroy_registered_window( w );
+		}
+    }else  /* if( required ) */
     {
         if( reuse_icon_canvas )
             canvas = asw->icon_canvas ;
@@ -469,8 +472,10 @@ LOCAL_DEBUG_OUT( "++CREAT Client(%lx(%s))->ICONT->canvas(%p)->window(%lx)", asw-
         }else
 			invalidate_canvas_config( canvas );
     }
+	
+	asw->icon_title_canvas = canvas;
 
-    return (asw->icon_title_canvas = canvas);
+    return canvas;
 }
 
 ASImage*
@@ -530,7 +535,6 @@ LOCAL_DEBUG_OUT( "++CREAT tbar(%p)->context(%s)", *tbar, context2text(context) )
 
         set_astbar_style( *tbar, BAR_STATE_FOCUSED, mystyle_name );
         set_astbar_style( *tbar, BAR_STATE_UNFOCUSED, "default" );
-        delete_astbar_tile( *tbar, -1 );
         if( img )
         {
             LOCAL_DEBUG_OUT("adding bar icon %p %ux%u", img, img->width, img->height );
@@ -573,10 +577,14 @@ grab_aswindow_buttons( ASWindow *asw, Bool focused )
 {
     if( asw )
     {
-        Bool do_focus_grab = (!focused && get_flags( Scr.Feel.flags, ClickToFocus ));
+        Bool do_focus_grab = (get_flags( Scr.Feel.flags, ClickToFocus ));
         if( do_focus_grab )
-            grab_focus_click( asw->frame );
-        else
+		{	
+            if( focused )
+				ungrab_focus_click( asw->frame );
+			else				
+				grab_focus_click( asw->frame );
+        }else
             ungrab_window_buttons( asw->frame );
 
         if( !ASWIN_GET_FLAGS(asw, AS_Dead) )
@@ -588,18 +596,29 @@ grab_aswindow_buttons( ASWindow *asw, Bool focused )
         if( asw->icon_canvas && !ASWIN_GET_FLAGS(asw, AS_Dead) && validate_drawable(asw->icon_canvas->w, NULL, NULL) != None )
         {
             ungrab_window_buttons( asw->icon_canvas->w );
-            grab_window_buttons( asw->icon_canvas->w, C_ICON );
             if( do_focus_grab )
-                grab_focus_click( asw->icon_canvas->w );
+			{
+	            if( focused )
+					ungrab_focus_click( asw->icon_canvas->w );
+				else				
+			        grab_focus_click( asw->icon_canvas->w );
+			}
+            grab_window_buttons( asw->icon_canvas->w, C_ICON );
         }
 
         if( asw->icon_title_canvas && asw->icon_title_canvas != asw->icon_canvas )
         {
             ungrab_window_buttons( asw->icon_title_canvas->w );
-            grab_window_buttons( asw->icon_title_canvas->w, C_ICON );
             if( do_focus_grab )
-                grab_focus_click( asw->icon_title_canvas->w );
-        }
+			{
+	            if( focused )
+					ungrab_focus_click( asw->icon_title_canvas->w );
+				else				
+	                grab_focus_click( asw->icon_title_canvas->w );
+			}
+            grab_window_buttons( asw->icon_title_canvas->w, C_ICON );
+    
+		}
     }
     XSync(dpy, False );
 }
@@ -819,8 +838,8 @@ estimate_titlebar_size( ASHints *hints, unsigned int *width_ret, unsigned int *h
         ASFlagType btn_mask ; 
 
 		btn_mask = compile_titlebuttons_mask (hints);
-        tbar->h_spacing = DEFAULT_TBAR_SPACING ;
-        tbar->v_spacing = DEFAULT_TBAR_SPACING ;
+        tbar->h_spacing = DEFAULT_TBAR_HSPACING ;
+        tbar->v_spacing = DEFAULT_TBAR_VSPACING ;
         /* left buttons : */
         add_astbar_btnblock(tbar, 0, 0, 0, NO_ALIGN,
                             &(Scr.Look.ordered_buttons[0]), btn_mask,
@@ -829,7 +848,7 @@ estimate_titlebar_size( ASHints *hints, unsigned int *width_ret, unsigned int *h
                             TBTN_ORDER_L2R );
       
 		/* label */
-        add_astbar_label(   tbar, 1, 0, 0, ALIGN_LEFT, DEFAULT_TBAR_SPACING, DEFAULT_TBAR_SPACING, hints->names[0], hints->names_encoding[0]);
+        add_astbar_label(   tbar, 1, 0, 0, ALIGN_LEFT, DEFAULT_TBAR_HSPACING, DEFAULT_TBAR_VSPACING, hints->names[0], hints->names_encoding[0]);
         /* right buttons : */
         add_astbar_btnblock(tbar, 2, 0, 0, NO_ALIGN,
                             &(Scr.Look.ordered_buttons[Scr.Look.button_first_right]), btn_mask,
@@ -880,7 +899,6 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
     char *mystyle_name = Scr.Look.MSWindow[BACK_FOCUSED]->name;
     char *frame_mystyle_name = NULL ;
     int *frame_contexts  = &(od->frame_contexts[0]);
-	Bool add_icon_label = (asw->icon_title==NULL) ;
 	Bool tbar_created = False;
 	Bool status_changed = False ;
 
@@ -892,7 +910,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 	/* 4) we need to prepare icon title window : */
 	check_icon_title_canvas( asw, (ASWIN_HFLAGS( asw, AS_IconTitle) &&
                                    get_flags(Scr.Feel.flags, IconTitle) &&
-       	                          !get_flags(Scr.Feel.flags, SuppressIcons)),
+								  !get_flags(Scr.Feel.flags, SuppressIcons)),
            	                      !get_flags(Scr.Look.flags, SeparateButtonTitle)&&
                	                  (asw->icon_canvas!=NULL) );
 
@@ -958,11 +976,12 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
 		{
 			ASImage *icon_image = get_window_icon_image( asw );
     		check_tbar( &(asw->icon_button), (asw->icon_canvas != NULL), AS_ICON_MYSTYLE,
-            		    icon_image, icon_image?icon_image->width:0, icon_image?icon_image->height:0,/* scaling icon image */
-                		0, ALIGN_CENTER,
-                		DEFAULT_TBAR_HILITE, DEFAULT_TBAR_HILITE,
+            		    icon_image, Scr.Look.ButtonWidth, Scr.Look.ButtonHeight,/* scaling icon image */
+                		0, Scr.Look.ButtonAlign,
+                		Scr.Look.ButtonBevel, Scr.Look.ButtonBevel,
                 		TEXTURE_TRANSPIXMAP_ALPHA, TEXTURE_TRANSPIXMAP_ALPHA,
                 		C_IconButton );
+			
 			if( icon_image )
     	    	safe_asimage_destroy( icon_image );
 		}
@@ -971,6 +990,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
     {
         set_astbar_style( asw->icon_button, BAR_STATE_UNFOCUSED, AS_ICON_MYSTYLE );
         set_astbar_style( asw->icon_button, BAR_STATE_FOCUSED, AS_ICON_MYSTYLE );
+		asw->icon_button->h_border = asw->icon_button->v_border = Scr.Look.ButtonIconSpacing ;
     }
 
     /* 7) now we have to create bar for icon title (optional) */
@@ -983,23 +1003,14 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
     
 	if( asw->icon_title )
     {
-		if( add_icon_label )
-		{
-	        LOCAL_DEBUG_OUT( "setting icon label to %s", ASWIN_ICON_NAME(asw) );
-	        add_astbar_label( asw->icon_title, 0, 0, 0, frame->title_align,
-							  DEFAULT_TBAR_SPACING, DEFAULT_TBAR_SPACING,
-						      ASWIN_ICON_NAME(asw),
-							  (asw->hints->icon_name_idx < 0)?AS_Text_ASCII :
-														  asw->hints->names_encoding[asw->hints->icon_name_idx]);
-		}else if( old_hints == NULL || mystrcmp(ASWIN_ICON_NAME(asw), old_hints->icon_name) != 0 )
-		{
-	        set_astbar_style( asw->icon_title, BAR_STATE_FOCUSED, AS_ICON_TITLE_MYSTYLE );
-  	    	set_astbar_style( asw->icon_title, BAR_STATE_UNFOCUSED, "default" );
-			change_astbar_first_label( asw->icon_title,
-									   ASWIN_ICON_NAME(asw),
-		  							  (asw->hints->icon_name_idx < 0)?AS_Text_ASCII :
-																	  asw->hints->names_encoding[asw->hints->icon_name_idx]);
-		}
+	    LOCAL_DEBUG_OUT( "setting icon label to %s", ASWIN_ICON_NAME(asw) );
+	    add_astbar_label( asw->icon_title, 0, 0, 0, frame->title_align,
+							DEFAULT_TBAR_HSPACING, DEFAULT_TBAR_VSPACING,
+						    ASWIN_ICON_NAME(asw),
+							(asw->hints->icon_name_idx < 0)?AS_Text_ASCII :
+							asw->hints->names_encoding[asw->hints->icon_name_idx]);
+	    set_astbar_style( asw->icon_title, BAR_STATE_FOCUSED, AS_ICON_TITLE_MYSTYLE );
+  	    set_astbar_style( asw->icon_title, BAR_STATE_UNFOCUSED, AS_ICON_TITLE_UNFOCUS_MYSTYLE );
     }
 
     if( asw->hints->mystyle_names[BACK_FOCUSED] )
@@ -1202,7 +1213,7 @@ hints2decorations( ASWindow *asw, ASHints *old_hints )
                           tbar_layout_col[ASO_TBAR_ELEM_LBL],
                           tbar_layout_row[ASO_TBAR_ELEM_LBL],
                           od->flip,
-                          title_align, DEFAULT_TBAR_SPACING, max( asw->tbar->top_bevel, asw->tbar->bottom_bevel),
+                          title_align, frame->title_h_spacing, frame->title_v_spacing,
                           asw->hints->names[0], asw->hints->names_encoding[0]);
 		
 			/* all the buttons go after the label to be rendered over it */
@@ -1269,7 +1280,7 @@ redecorate_window( ASWindow *asw, Bool free_resources )
 {
 	int i ;
 
-LOCAL_DEBUG_OUT( "asw(%p)->free_res(%d)", asw, free_resources );
+LOCAL_DEBUG_OUT( "as?w(%p)->free_res(%d)", asw, free_resources );
     if( AS_ASSERT(asw) )
         return ;
 
@@ -1288,8 +1299,12 @@ LOCAL_DEBUG_OUT( "asw(%p)->free_res(%d)", asw, free_resources );
         check_side_canvas( asw, FR_S, False );
         check_side_canvas( asw, FR_N, False );
 
-        check_icon_title_canvas( asw, False, False );
-        check_icon_canvas( asw, False );
+		if( asw->icon_canvas == asw->icon_title_canvas )
+			asw->icon_title_canvas = NULL ;
+		if( asw->icon_title_canvas && asw->icon_title_canvas->w )
+	        check_icon_title_canvas( asw, False, False );
+		if( asw->icon_canvas && asw->icon_canvas->w )
+	        check_icon_canvas( asw, False );
         check_client_canvas( asw, False );
         check_frame_canvas( asw, False );
 
