@@ -148,7 +148,7 @@ WaitForButtonsUpLoop ()
 }
 
 Bool
-WaitEventLoop( ASEvent *event, int finish_event_type, long timeout )
+WaitEventLoop( ASEvent *event, int finish_event_type, long timeout, ASHintWindow *hint )
 {
 	unsigned long mask = ButtonPressMask | ButtonReleaseMask |
 						 ExposureMask | KeyPressMask | ButtonMotionMask |
@@ -177,6 +177,8 @@ WaitEventLoop( ASEvent *event, int finish_event_type, long timeout )
 		}
         DigestEvent( event );
         DispatchEvent( event, done );
+		if( event->x.type == MotionNotify && hint ) 
+			update_ashint_geometry( hint, False );
     }
 
     return True ;
@@ -236,6 +238,8 @@ WaitWindowLoop( char *pattern, long timeout )
 	time_t click_end_time = end_time / 4;
 	time_t start_time = time(NULL);
 	ASWindow *asw = NULL ;
+	ASHintWindow *hint ;
+	char *text ;
 
 	end_time += start_time ;
 	click_end_time += start_time ;
@@ -247,6 +251,9 @@ WaitWindowLoop( char *pattern, long timeout )
     if( (asw = complex_pattern2ASWindow( pattern )) != NULL )
   		return asw;
 
+	text = safemalloc(64+strlen(pattern)+1);
+	sprintf( text, "Waiting for window matching \"%s\" ... Press button to cancel.", pattern );
+	hint = create_ashint_window( ASDefaultScr, &(Scr.Look), text);
 	while (!done)
 	{
 		do
@@ -265,11 +272,15 @@ WaitWindowLoop( char *pattern, long timeout )
 					{
 						end_time = click_end_time ;
 						break;
-					}
-
-                    if( (event.x.type == MapNotify || event.x.type == PropertyNotify )&& event.client )
+					}else if( event.x.type == MotionNotify && hint ) 
+						update_ashint_geometry( hint, False );
+                    
+					if( (event.x.type == MapNotify || event.x.type == PropertyNotify )&& event.client )
                         if( (asw = complex_pattern2ASWindow( pattern )) != NULL )
+						{
+							destroy_ashint_window( &hint );
                             return asw;
+						}
                 }
 			}
 			if( time(NULL) > end_time )
@@ -284,6 +295,7 @@ WaitWindowLoop( char *pattern, long timeout )
 			break;
 		afterstep_wait_pipes_input ();
 	}
+	destroy_ashint_window( &hint );
     return NULL;
 }
 
@@ -366,6 +378,7 @@ DigestEvent( ASEvent *event )
 		int pointer_root_x = xk->x_root;
 		int pointer_root_y = xk->y_root;
 		static int last_pointer_root_x = -1, last_pointer_root_y = -1; 
+		int tbar_side = ASWIN_HFLAGS(asw, AS_VerticalTitle)?FR_W:FR_N ; 
 
         /* Since key presses and button presses are grabbed in the frame
          * when we have re-parented windows, we need to find out the real
@@ -407,7 +420,11 @@ DigestEvent( ASEvent *event )
 
 			}
 
-            if( w != asw->frame )
+	        if (ASWIN_GET_FLAGS(asw, AS_Shaded) && canvas != asw->frame_sides[tbar_side] )
+			{
+				event->context = C_NO_CONTEXT ;				
+				XRaiseWindow( dpy, asw->frame_sides[tbar_side]->w );
+			}else if( w != asw->frame )
             {
                 if( event->w == asw->frame )
                 {
@@ -439,6 +456,11 @@ DigestEvent( ASEvent *event )
                         }
                 }
             }
+			if( event->context == C_NO_CONTEXT && get_flags(Scr.Feel.flags, ClickToFocus) )
+			{
+				w = asw->frame ; 
+				event->context = C_FRAME ;
+			}	 
             event->w = w ;
         }else
         {
@@ -464,14 +486,16 @@ DigestEvent( ASEvent *event )
             }
         }
 
-        on_astbar_pointer_action( pointer_bar, event->context, 
-								  (event->x.type == LeaveNotify),
-								  (last_pointer_root_x != pointer_root_x || last_pointer_root_y != pointer_root_y) );
-		
+		if( pointer_bar != NULL ) 
+		{	
+        	on_astbar_pointer_action( pointer_bar, event->context, 
+								  	(event->x.type == LeaveNotify),
+								  	(last_pointer_root_x != pointer_root_x || last_pointer_root_y != pointer_root_y) );
+		}		
 		last_pointer_root_x = pointer_root_x ;
 		last_pointer_root_y = pointer_root_y ;
 
-		if( asw != NULL && w != asw->w && w != asw->frame )
+		if( asw != NULL && w != asw->w && w != asw->frame && event->context != C_NO_CONTEXT )
 			apply_context_cursor( w, &(Scr.Feel), event->context );
         event->widget  = canvas ;
         /* we have to do this at all times !!!! */
@@ -698,30 +722,39 @@ HandleKeyPress ( ASEvent *event )
 	FuncKey      *key;
     XKeyEvent *xk = &(event->x.xkey);
     unsigned int modifier = (xk->state & nonlock_mods);
+	int m ; 
 
 	/* Here's a real hack - some systems have two keys with the
 		* same keysym and different keycodes. This converts all
 		* the cases to one keycode. */
-    xk->keycode = XKeysymToKeycode (dpy, XKeycodeToKeysym (dpy, xk->keycode, 0));
+	for( m = 0 ; m < 8 ; ++m ) 
+	{	
+		KeySym keysym = XKeycodeToKeysym (dpy, xk->keycode, m); 
+		int keycode ; 
+		if( keysym == NoSymbol ) 
+			continue;
+    	if( (keycode = XKeysymToKeycode (dpy, keysym)) == 0 )
+			continue;
+		xk->keycode = keycode;
 
-    for (key = Scr.Feel.FuncKeyRoot; key != NULL; key = key->next)
-	{
-        if ((key->keycode == xk->keycode) &&
-			((key->mods == (modifier & (~LockMask))) ||
-             (key->mods == AnyModifier)) && (key->cont & event->context))
+    	for (key = Scr.Feel.FuncKeyRoot; key != NULL; key = key->next)
 		{
-			/* check if the warp key was pressed */
-            ExecuteFunction (key->fdata, event, -1);
-			return;
+        	if ((key->keycode == xk->keycode) &&
+				((key->mods == (modifier & (~LockMask))) ||
+             	(key->mods == AnyModifier)) && (key->cont & event->context))
+			{
+				/* check if the warp key was pressed */
+            	ExecuteFunction (key->fdata, event, -1);
+				return;
+			}
 		}
 	}
-
 	/* if a key has been pressed and it's not one of those that cause
 	   warping, we know the warping is finished */
     if( get_flags( AfterStepState, ASS_WarpingMode ) )
         EndWarping();
 
-	LOCAL_DEBUG_OUT( "client = %p", event->client );
+	LOCAL_DEBUG_OUT( "client = %p, context = %s", event->client, context2text((event)->context));
 	/* if we get here, no function key was bound to the key.  Send it
      * to the client if it was in a window we know about: */
     if (event->client)
@@ -821,7 +854,9 @@ HandlePropertyNotify (ASEvent *event)
 										Database,
                                         asw->hints, asw->status ) )
         {
-            show_debug( __FILE__, __FUNCTION__, __LINE__, "New name is \"%s\", icon_name \"%s\", following title change ? %s", 
+            if( ASWIN_GET_FLAGS( asw, AS_Dead ) )
+				return;
+			show_debug( __FILE__, __FUNCTION__, __LINE__, "New name is \"%s\", icon_name \"%s\", following title change ? %s", 
 				        ASWIN_NAME(asw), ASWIN_ICON_NAME(asw), get_flags( Scr.Feel.flags, FollowTitleChanges)?"yes":"no" );
 	    	LOCAL_DEBUG_OUT( "hints flags = %lX, ShortLived ? %lX ", asw->hints->flags, ASWIN_HFLAGS( asw, AS_ShortLived ) );
 			if( old_name && strcmp( old_name, ASWIN_NAME(asw) ) != 0 )
@@ -1067,33 +1102,43 @@ HandleButtonPress ( ASEvent *event, Bool deffered )
 	Bool 		  raise_on_click = False ; 
     Bool 		  focus_accepted = False ;
     Bool 		  eat_click = False;
+	Bool 		  activate_window = False ;
 
 	/* click to focus stuff goes here */
     if( asw != NULL )
     {
+		LOCAL_DEBUG_OUT( "deferred = %d, button = %X", deffered, (event->context&(~C_TButtonAll)) );
         /* if all we do is pressing titlebar buttons - then we should not raise/focus window !!! */
-        if( !deffered && (event->context&(~C_TButtonAll)) != 0 )
+        if( !deffered )
         {
-            if (get_flags( Scr.Feel.flags, ClickToFocus) )
-      		{
-          		if ( asw != Scr.Windows->ungrabbed && (xbtn->state & nonlock_mods) == 0)
-                {
-                    if( Scr.Windows->focused != asw )
-                        focus_accepted = activate_aswindow( asw, False, False);
-                    if( focus_accepted && get_flags( Scr.Feel.flags, EatFocusClick ) )
-                        eat_click = True ;
-                }
-	        }
+			if( (event->context&(~C_TButtonAll)) != 0 )
+			{	
+            	if (get_flags( Scr.Feel.flags, ClickToFocus) )
+      			{
+					LOCAL_DEBUG_OUT( "asw = %p, ungrabbed = %p, nonlock_mods = %x", asw, Scr.Windows->ungrabbed, (xbtn->state & nonlock_mods) );
+          			if ( asw != Scr.Windows->ungrabbed && (xbtn->state & nonlock_mods) == 0)
+                	{
+						if( get_flags( Scr.Feel.flags, EatFocusClick ) )
+						{
+							if( Scr.Windows->focused != asw )
+								if( (focus_accepted = activate_aswindow( asw, False, False)) )
+                        			eat_click = True ;
+						}else if( Scr.Windows->focused != asw )
+							activate_window = True;
+						LOCAL_DEBUG_OUT( "eat_click = %d", eat_click );
+                	}
+	        	}
 
-            if ( get_flags(Scr.Feel.flags, ClickToRaise) )
-				raise_on_click = ( Scr.Feel.RaiseButtons == 0 || (Scr.Feel.RaiseButtons & (1 << xbtn->button))) ; 
-
-            if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
-  		    {
-      		    XSync (dpy, 0);
-          		XAllowEvents (dpy, (event->context == C_WINDOW) ? ReplayPointer : AsyncPointer, CurrentTime);
-	            XSync (dpy, 0);
-  		    }
+            	if ( get_flags(Scr.Feel.flags, ClickToRaise) )
+					raise_on_click = ( Scr.Feel.RaiseButtons == 0 || (Scr.Feel.RaiseButtons & (1 << xbtn->button))) ; 
+			}
+			
+			if (!ASWIN_GET_FLAGS(asw, AS_Iconic))
+  			{
+      			XSync (dpy, 0);
+        		XAllowEvents (dpy, (event->context == C_WINDOW) ? ReplayPointer : AsyncPointer, CurrentTime);
+	    		XSync (dpy, 0);
+  			}
 		} /* !deffered */
 
         press_aswindow( asw, event->context );
@@ -1128,6 +1173,10 @@ HandleButtonPress ( ASEvent *event, Bool deffered )
 		}
 	}
 
+	LOCAL_DEBUG_OUT( "ashandled = %d, context = %X", AShandled, (event->context&(C_CLIENT|C_TITLE)) );
+	if( activate_window && (!AShandled || (event->context&(C_CLIENT|C_TITLE)))) 
+		focus_accepted = activate_aswindow( asw, False, False);
+
 	if( raise_on_click && (asw->internal == NULL || (event->context&C_CLIENT) == 0 ) )
     	restack_window(asw,None,focus_accepted?Above:TopIf);
 
@@ -1205,7 +1254,7 @@ HandleEnterNotify (ASEvent *event)
 
     if (ASWIN_HFLAGS(asw,AS_AcceptsFocus))
 	{
-        if (!get_flags(Scr.Feel.flags, ClickToFocus))
+        if (!get_flags(Scr.Feel.flags, ClickToFocus) || asw->internal != NULL )
 		{
             if (Scr.Windows->focused != asw)
                 activate_aswindow(asw, False, False);
@@ -1265,6 +1314,8 @@ HandleConfigureRequest ( ASEvent *event )
 	 * to configuration requests for windows which have never been mapped.
 	 */
 
+	LOCAL_DEBUG_OUT( "cre={0x%lx, geom = %dx%d%+d%+d} ", cre->value_mask, cre->width, cre->height,
+	cre->x, cre->y );
     if (asw == NULL)
 	{
         unsigned long xwcm;
@@ -1279,6 +1330,13 @@ HandleConfigureRequest ( ASEvent *event )
 		return;
 	}
 
+	if( ASWIN_HFLAGS(asw, AS_IgnoreConfigRequest ) )
+	{	
+		LOCAL_DEBUG_OUT( "Ignoring ConfigureRequest for client %p as required by hints", asw );
+		send_canvas_configure_notify(asw->frame_canvas, asw->client_canvas);
+		return;
+	}
+
     if (cre->value_mask & CWStackMode)
         restack_window( asw, (cre->value_mask & CWSibling)?cre->above:None, cre->detail );
 
@@ -1383,7 +1441,7 @@ void HandleShmCompletion(ASEvent *event)
 {
 #ifdef XSHMIMAGE
     XShmCompletionEvent  *sev = (XShmCompletionEvent*) &(event->x);
-	LOCAL_DEBUG_OUT( "XSHMIMAGE> EVENT : offset   %ld(%lx)", (long)sev->offset, (unsigned long)(sev->offset) );
+	LOCAL_DEBUG_OUT( "XSHMIMAGE> EVENT : offset   %ld(%lx), shmseg = %lx", (long)sev->offset, (unsigned long)(sev->offset), sev->shmseg );
 	if( !is_background_xfer_ximage( sev->shmseg ) )
 		destroy_xshm_segment( sev->shmseg );
 #endif /* SHAPE */
