@@ -179,11 +179,11 @@ ReadModuleInput (module_t *module, size_t * offset, size_t size, void *ptr)
 static void
 CheckCmdTextSize (module_t *module, CARD32 * size, CARD32 * curr_len, char **text)
 {
-	/* max command length is 255 */
-	if (*size > 255)
+	/* max command length is 1024 */
+	if (*size > 1024)
 	{
         show_error("command from module '%s' is too big (%d)", module->name, *size);
-		*size = 255;
+		*size = 1024;
 	}
 
 	/* need to be able to read in command */
@@ -260,7 +260,7 @@ LOCAL_DEBUG_OUT("Incoming message in proto 1%s","");
 LOCAL_DEBUG_OUT("Incoming message in proto 2%s","");
             if (pfunc == NULL)
 			{
-				pfunc = (FunctionData *) safemalloc (sizeof (FunctionData));
+				pfunc = (FunctionData *) safecalloc (1, sizeof (FunctionData));
 				init_func_data (pfunc);
 				ibuf->func = pfunc;
 			}
@@ -721,6 +721,32 @@ KillModuleByName (char *name)
 	return;
 }
 
+int FindModuleByName (char *name)
+{
+    wild_reg_exp  *wrexp = compile_wild_reg_exp( name );
+	int module = -1;
+
+    if ( wrexp != NULL && Modules)
+    {
+        register int i = MODULES_NUM;
+        register module_t *list = MODULES_LIST ;
+
+        while( --i >= 0 )
+            if (list[i].fd > 0)
+            {
+                LOCAL_DEBUG_OUT( "checking if module %d \"%s\" matches regexp \"%s\"", i, list[i].name, name);
+                if (match_wild_reg_exp( list[i].name, wrexp ) == 0 )
+				{	
+                    module = i ; 
+					break;
+				}
+            }
+        destroy_wild_reg_exp( wrexp );
+    }
+	return module;
+}
+
+
 void
 SendPacket ( int channel, send_data_type msg_type, send_data_type num_datum, ...)
 {
@@ -930,6 +956,7 @@ RunCommand (FunctionData * fdata, unsigned int channel, Window w)
 	 default:
         {
             ASEvent event;
+			Bool defered = False ;
             memset( &event, 0x00, sizeof(ASEvent) );
             event.w = w;
             if ((event.client = window2ASWindow(w)) == NULL )
@@ -942,6 +969,7 @@ RunCommand (FunctionData * fdata, unsigned int channel, Window w)
                 event.x.xbutton.x_root = event.client->frame_canvas->root_x+1;
                 event.x.xbutton.y_root = event.client->frame_canvas->root_y+1;
             }
+			
             event.x.xany.type = ButtonRelease;
             event.x.xbutton.button = 1;
             event.x.xbutton.x = 0;
@@ -951,7 +979,9 @@ RunCommand (FunctionData * fdata, unsigned int channel, Window w)
             event.scr = ASDefaultScr ;
             event.event_time = Scr.last_Timestamp ;
             /* there must be no deffering on module commands ! */
-            ExecuteFunctionExt (fdata, &event, channel, True);
+
+			defered = ( w != None || !IsWindowFunc (fdata->func) );
+            ExecuteFunctionExt (fdata, &event, channel, defered);
         }
     }
     free_func_data (fdata);
