@@ -673,14 +673,15 @@ release_old_background( int desk, Bool forget )
     {
         if( Scr.RootImage == im )
         {                                      /* always do that ! */
-            safe_asimage_destroy( im );
             Scr.RootImage = NULL ;
-			im = NULL ;
-        }else if( forget || im->width*im->height >= Scr.Look.KillBackgroundThreshold )
+            if( safe_asimage_destroy( im ) <= 0 )
+				im = NULL ;
+        } 
+		if( im != NULL && (forget || im->width*im->height >= Scr.Look.KillBackgroundThreshold) )
         {
             LOCAL_DEBUG_OUT( "im = %p, ref_count = %d", im, im->ref_count );
-            safe_asimage_destroy( im );
-			im = NULL ;
+            if( safe_asimage_destroy( im ) <= 0 )
+				im = NULL ;
         }else if( strcmp( im->name, imname) != 0 )	 /* we need to store it for future use ! */
 		{	                                     	 /* for future references we store image under imname */
 			if( im->ref_count > 1 )
@@ -696,7 +697,6 @@ release_old_background( int desk, Bool forget )
 			free( back->loaded_im_name );
 			back->loaded_im_name = NULL ; 
 		}
-
 		if( im )
 		{	
 			back->loaded_im_name = mystrdup(im->name); 
