@@ -1,4 +1,4 @@
-/*
+/* 
  * Copyright (c) 2002,2003 Sasha Vasko <sasha@aftercode.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -320,6 +320,8 @@ move_resize_longbar( ASTBarData *bar, ASCanvas *canvas, ASOrientation *od,
     unsigned int h = bar->height;
     int bar_size ;
 
+	LOCAL_DEBUG_OUT( "normal_offset = %d, normal_length = %d, corner_sizes = %d,%d, vert = %d, force = %d", 
+					 normal_offset, normal_length, corner_size1, corner_size2, vertical, force_render );
     /* swapping bar height in case of vertical orientation of the etire window: */
     *(od->in_width) = w ;
     *(od->in_height) = h ;
@@ -369,7 +371,8 @@ condense_tbar( ASTBarData *tbar, unsigned int max_size, unsigned int *off1, unsi
                         *off1 /= 2 ;
                         *off2 = *off1 ;
                     }    
-                }
+                }else
+					*off2 = max_size - condensed ;
             }else 
 				condensed = max_size ;
         }else
@@ -385,7 +388,8 @@ condense_tbar( ASTBarData *tbar, unsigned int max_size, unsigned int *off1, unsi
                         *off1 /= 2 ;
                         *off2 = *off1 ;
                     }    
-                }
+                }else
+					*off2 = max_size - condensed ;
             }else 
 				condensed = max_size ;
         }    
@@ -428,7 +432,7 @@ move_resize_frame_bars( ASWindow *asw, int side, ASOrientation *od, unsigned int
             set_flags( canvas->state, CANVAS_FORCE_MASK );
 
 		/* title always considered a "horizontal bar" */
-        if( move_resize_longbar( title, canvas, od, 0, tbar_width, tbar_offset1, tbar_offset2, False, force_render ) )
+		if( move_resize_longbar( title, canvas, od, 0, normal_width, tbar_offset1, tbar_offset2, False, force_render ) )
             rendered = True;
         tbar_size = *(od->in_height);
     }
@@ -467,6 +471,31 @@ move_resize_frame_bars( ASWindow *asw, int side, ASOrientation *od, unsigned int
     return rendered;
 }
 
+static void
+resize_frame_subwindows( ASWindow *asw, ASOrientation *od, unsigned int frame_win_width, unsigned int frame_win_height ) 
+{
+    register unsigned int *frame_size = &(asw->status->frame_size[0]) ;
+    unsigned int normal_width, normal_height ;
+
+	if( od == NULL ) 
+		od = get_orientation_data(asw);
+
+    *(od->in_width)  = asw->frame_canvas->width ;
+    *(od->in_height) = asw->frame_canvas->height ;
+    normal_width  = *(od->out_width)  ;
+    normal_height = *(od->out_height) ;
+				
+	resize_canvases( asw, od, normal_width, normal_height, frame_size );
+    if( !ASWIN_GET_FLAGS(asw, AS_Shaded ) )  /* leave shaded client alone ! */
+    	moveresize_canvas( asw->client_canvas,
+                           frame_size[FR_W],
+                           frame_size[FR_N],
+                           (int)frame_win_width-(int)(frame_size[FR_W]+frame_size[FR_E]),
+                           (int)frame_win_height-(int)(frame_size[FR_N]+frame_size[FR_S]));
+}
+
+
+
 Bool
 apply_window_status_size(register ASWindow *asw, ASOrientation *od)
 {
@@ -502,6 +531,10 @@ LOCAL_DEBUG_OUT( "**CONFG Client(%lx(%s))->status(%ux%u%+d%+d,%s,%s(%d>-%d))",
 					asw->frame_canvas->root_y != asw->status->y ||
 					asw->frame_canvas->width != new_width ||
 					asw->frame_canvas->height != new_height );
+
+		if( step_size <= 0 )
+			resize_frame_subwindows( asw, od, new_width, new_height ); 
+
         moveresize_canvas(  asw->frame_canvas, asw->status->x, asw->status->y,
                             new_width, new_height );
     }
@@ -629,6 +662,22 @@ check_frame_side_config( ASWindow *asw, Window w, ASOrientation *od )
 	return found;
 }
 
+static void 
+move_shading_frame( ASWindow *asw, ASOrientation *od, int step_size )
+{
+	if( asw->frame_sides[od->sbar_side] )
+	{
+		XRaiseWindow( dpy, asw->frame_sides[od->sbar_side]->w );
+		if( ASWIN_HFLAGS(asw, AS_VerticalTitle) )
+			move_canvas( asw->frame_sides[od->sbar_side], step_size - asw->frame_sides[od->sbar_side]->width, 0 );
+		else
+			move_canvas( asw->frame_sides[od->sbar_side], 0, step_size - asw->frame_sides[od->sbar_side]->height );
+	}
+	if( asw->frame_sides[od->tbar_side] )
+		XRaiseWindow( dpy, asw->frame_sides[od->tbar_side]->w );
+}
+
+
 /* this gets called when StructureNotify/SubstractureNotify arrives : */
 void
 on_window_moveresize( ASWindow *asw, Window w )
@@ -676,26 +725,33 @@ LOCAL_DEBUG_OUT( "changes=0x%X", changes );
 			LOCAL_DEBUG_OUT( "step_size = %d", step_size );
             if( step_size <= 0 )  /* don't moveresize client window while shading !!!! */
             {
-                resize_canvases( asw, od, normal_width, normal_height, frame_size );
-                if( !ASWIN_GET_FLAGS(asw, AS_Shaded ) )  /* leave shaded client alone ! */
-                    moveresize_canvas( asw->client_canvas,
-                                    frame_size[FR_W],
-                                    frame_size[FR_N],
-                                    asw->frame_canvas->width-(frame_size[FR_W]+frame_size[FR_E]),
-                                    asw->frame_canvas->height-(frame_size[FR_N]+frame_size[FR_S]));
-            }else
+				resize_frame_subwindows( asw, od, 	asw->frame_canvas->width,
+								  					asw->frame_canvas->height ); 					
+	        }else
 			{ 
 				if( normal_height != step_size )
             	{
-                	sleep_a_millisec(10);
-                	/* we get smoother animation if we move decoration ahead of actually
-                 	* resizing frame window : */
-                	resize_canvases( asw, od, normal_width, step_size, frame_size );
                 	*(od->in_width)=normal_width ;
                 	*(od->in_height)=step_size ;
+			     	
+					if( normal_height < step_size )           	
+					{	
+						/* we get smoother animation if we move decoration ahead of actually
+                 		 * resizing frame window : */
+ 						move_shading_frame( asw, od, step_size );
+						ASSync( False );
+                		sleep_a_millisec(10);
 	/*LOCAL_DEBUG_OUT( "**SHADE Client(%lx(%s))->(%d>-%d)", asw->w, ASWIN_NAME(asw)?ASWIN_NAME(asw):"noname", asw->shading_steps, step_size );*/
-                	resize_canvas( asw->frame_canvas, *(od->out_width), *(od->out_height));
-                	ASSync(False);
+                		resize_canvas( asw->frame_canvas, *(od->out_width), *(od->out_height));
+                		ASSync(False);
+					}else
+					{	
+ 						sleep_a_millisec(10);
+                		resize_canvas( asw->frame_canvas, *(od->out_width), *(od->out_height));
+                		ASSync(False);
+						move_shading_frame( asw, od, step_size );
+						ASSync( False );
+					}	
             	}else
 				{  /* probably just the change of the look - titlebar may need to be resized */
 					resize_canvases( asw, od, normal_width, normal_height, frame_size );
@@ -782,6 +838,8 @@ on_window_title_changed( ASWindow *asw, Bool update_display )
 {
     if( AS_ASSERT(asw) )
         return ;
+    if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
+		print_clean_hints( NULL, NULL, asw->hints );
     if( asw->tbar )
     {
         ASCanvas *canvas = ASWIN_HFLAGS(asw, AS_VerticalTitle)?asw->frame_sides[FR_W]:asw->frame_sides[FR_N];
@@ -806,7 +864,8 @@ on_window_title_changed( ASWindow *asw, Bool update_display )
 					SetShape( asw, 0 );
             }
     }
-    if( asw->icon_title )
+    LOCAL_DEBUG_OUT( "icon_title = %p, icon_name = \"%s\"", asw->icon_title, ASWIN_ICON_NAME(asw)?ASWIN_ICON_NAME(asw):"(null)" );
+	if( asw->icon_title )
     {
         if( change_astbar_first_label( asw->icon_title, ASWIN_ICON_NAME(asw), (asw->hints->icon_name_idx<0)?AS_Text_ASCII:asw->hints->names_encoding[asw->hints->icon_name_idx] ) )
             on_icon_changed( asw );
@@ -827,7 +886,7 @@ on_window_hints_changed( ASWindow *asw )
         return ;
     if( is_output_level_under_threshold(OUTPUT_LEVEL_HINTS) )
         print_hints( NULL, NULL, &raw_hints );
-    hints = merge_hints( &raw_hints, Database, NULL, Scr.Look.supported_hints, HINT_ANY, NULL );
+    hints = merge_hints( &raw_hints, Database, NULL, Scr.Look.supported_hints, HINT_ANY, NULL, asw->w );
     destroy_raw_hints( &raw_hints, True );
     if( hints )
     {
@@ -861,10 +920,11 @@ on_window_hints_changed( ASWindow *asw )
 	if( hints2decorations( asw, old_hints ) )
 	    on_window_status_changed( asw, False, True );
 
+	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 || 
+		mystrcmp( old_hints->res_class, hints->res_class ) != 0)
+  		broadcast_res_names( asw );
 	if( mystrcmp( old_hints->names[0], hints->names[0] ) != 0 )
 	    broadcast_window_name( asw );
-	if( mystrcmp( old_hints->res_name, hints->res_name ) != 0 )
-  		broadcast_res_names( asw );
 	if( mystrcmp( old_hints->icon_name, hints->icon_name ) != 0 )
 	    broadcast_icon_name( asw );
 
@@ -880,7 +940,9 @@ update_window_tbar_size(	ASWindow *asw )
         unsigned int tbar_width = 0 ;
         unsigned int tbar_height = 0 ;
         int x_offset = 0, y_offset = 0 ;
-        if( ASWIN_HFLAGS(asw, AS_VerticalTitle) )
+        LOCAL_DEBUG_OUT( "IsVertical = %lX", ASWIN_HFLAGS(asw, AS_VerticalTitle) );
+		
+		if( ASWIN_HFLAGS(asw, AS_VerticalTitle) )
         {
             tbar_size = calculate_astbar_width( asw->tbar );
             tbar_width = tbar_size ; 
@@ -1128,20 +1190,6 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s focused)", asw, focused?"":"not" );
 			LOCAL_DEBUG_OUT( "canvas save swapped for side %d", i );
 			update_canvas = NULL ;
 		}
-#if 0				  
-		if( focused )
-		{
-			save_canvas(update_canvas);
-		}else
-		{
-			if( restore_canvas(asw->frame_sides[i]) )
-			{
-				LOCAL_DEBUG_OUT( "canvas restored for side %d", i );
-				update_canvas = NULL ;
-			}
-		}
-#endif
-
 		if( i == od->tbar_side )
 			set_astbar_focused( asw->tbar, update_canvas, focused );
 
@@ -1157,7 +1205,7 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s focused)", asw, focused?"":"not" );
         asw->internal->on_hilite_changed( asw->internal, NULL, focused );
     if( ASWIN_GET_FLAGS( asw, AS_ShapedDecor ) )
         SetShape( asw, 0 );
-    if(!ASWIN_GET_FLAGS(asw, AS_Iconic))
+    if(ASWIN_GET_FLAGS(asw, AS_Iconic))
 	{	
         set_astbar_focused( asw->icon_button, asw->icon_canvas, focused );
         set_astbar_focused( asw->icon_title, asw->icon_title_canvas, focused );
@@ -1189,7 +1237,9 @@ LOCAL_DEBUG_CALLER_OUT( "(%p,%s)", asw, context2text(pressed_context));
         /* now posting all the changes on display :*/
         for( i = FRAME_SIDES; --i >= 0; )
             if( is_canvas_dirty(asw->frame_sides[i]) )
+			{
                 update_canvas_display( asw->frame_sides[i] );
+			}
         if( asw->internal && asw->internal->on_pressure_changed )
             asw->internal->on_pressure_changed( asw->internal, pressed_context&C_CLIENT );
     }else /* Iconic !!! */
@@ -1224,6 +1274,7 @@ save_aswindow_anchor( ASWindow *asw, Bool hor, Bool vert )
 void
 moveresize_aswindow_wm( ASWindow *asw, int x, int y, unsigned int width, unsigned int height, Bool save_anchor )
 {
+	LOCAL_DEBUG_CALLER_OUT( "asw(%p)->geom(%dx%d%+d%+d)->save_anchor(%d)", asw, x, y, width, height, save_anchor );
     if( !AS_ASSERT(asw) )
     {
         ASStatusHints scratch_status  = *(asw->status);
@@ -1316,6 +1367,38 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
 	}
 	LOCAL_DEBUG_OUT( "status->pos = %+d%+d, Scr.Vpos = %+d%+d", t->status->x, t->status->y, Scr.Vx, Scr.Vy );
 
+	if( get_flags (t->hints->flags, AS_MinSize) ) 
+	{	
+		int width = t->status->width ;
+		int height = t->status->height ;
+		if( (!get_flags (t->status->flags, AS_StartSizeUser) && width < t->hints->min_width) || width == 1 )
+			width = min( t->hints->min_width, Scr.MyDisplayWidth );
+		if( (!get_flags (t->status->flags, AS_StartSizeUser) && height < t->hints->min_height) || height == 1 )
+			width = min( t->hints->min_height, Scr.MyDisplayHeight );
+		if( width != t->status->width || height != t->status->height ) 
+		{
+			int dx = 0, dy = 0 ;
+			if( t->hints->gravity == EastGravity || 
+				t->hints->gravity == SouthEastGravity || 
+				t->hints->gravity == NorthEastGravity ||
+				t->hints->gravity == CenterGravity )
+				dx = (int)(t->status->width) - width ; 
+			if( t->hints->gravity == SouthGravity || 
+				t->hints->gravity == SouthEastGravity || 
+				t->hints->gravity == SouthWestGravity ||
+				t->hints->gravity == CenterGravity)
+				dy = (int)(t->status->height) - height ; 
+			if( t->hints->gravity == EastGravity || t->hints->gravity == CenterGravity ) 
+				dx = dx/2 ;
+			if( t->hints->gravity == SouthGravity || t->hints->gravity == CenterGravity ) 
+				dy = dy/2 ;
+			t->status->x += dx ;			
+			t->status->y += dy ;			   
+			t->status->width = width ;			   
+			t->status->height = height ;			   
+			XResizeWindow( dpy, t->w, width, height );
+		}	 
+	}
     if( !get_flags(AfterStepState, ASS_NormalOperation) )
     {
         int min_x, min_y, max_x, max_y ;
@@ -1414,6 +1497,7 @@ init_aswindow_status( ASWindow *t, ASStatusHints *status )
         validate_window_anchor( t, &(t->anchor) );    
         anchor2status ( t->status, t->hints, &(t->anchor));
     }    
+
     return pending_placement;
 }
 
@@ -1471,10 +1555,6 @@ LOCAL_DEBUG_CALLER_OUT( "client = %p, iconify = %d", asw, iconify );
         else if( asw->icon_title_canvas )
             asw->status->icon_window = asw->icon_title_canvas->w ;
 
-		LOCAL_DEBUG_OUT( "hilited == %p", Scr.Windows->hilited );
-    	if( Scr.Windows->hilited == asw )
-			hide_hilite();
-
         if (get_flags(Scr.Feel.flags, ClickToFocus) || get_flags(Scr.Feel.flags, SloppyFocus))
         {
             if (asw == Scr.Windows->focused)
@@ -1496,8 +1576,14 @@ LOCAL_DEBUG_OUT( "unmaping client window 0x%lX", (unsigned long)asw->w );
                     complete_wm_state_transition( asw, IconicState );
                 }
             }
+			ASSync(False);
         }
-        /* finally mapping the icon windows : */
+		
+		LOCAL_DEBUG_OUT( "hilited == %p", Scr.Windows->hilited );
+    	if( Scr.Windows->hilited == asw )
+			hide_hilite();
+        
+		/* finally mapping the icon windows : */
         add_iconbox_icon( asw );
         restack_window( asw, None, Below );
         map_canvas_window(asw->icon_canvas, True );
@@ -1518,8 +1604,7 @@ LOCAL_DEBUG_OUT( "updating status to iconic for client %p(\"%s\")", asw, ASWIN_N
             if( asw->icon_canvas != asw->icon_title_canvas )
                 unmap_canvas_window(asw->icon_title_canvas );
 
-            ASWIN_DESK(asw) = get_flags(Scr.Feel.flags, StubbornIcons)?asw->DeIconifyDesk:Scr.CurrentDesk;
-            quietly_reparent_aswindow( asw, (ASWIN_DESK(asw)==Scr.CurrentDesk)?Scr.Root:Scr.ServiceWin, True );
+			change_aswindow_desktop( asw, get_flags(Scr.Feel.flags, StubbornIcons)?asw->DeIconifyDesk:Scr.CurrentDesk, True );
         }
 
         asw->status->icon_window = None ;
@@ -1712,12 +1797,15 @@ LOCAL_DEBUG_OUT( "changing window's layer to %d", layer );
     }
 }
 
-void change_aswindow_desktop( ASWindow *asw, int new_desk )
+void change_aswindow_desktop( ASWindow *asw, int new_desk, Bool force )
 {
     int old_desk ;
     if( AS_ASSERT(asw) )
         return ;
-    if( ASWIN_DESK(asw) == new_desk || ASWIN_GET_FLAGS(asw, AS_Sticky))
+	if( ASWIN_GET_FLAGS(asw, AS_Sticky) ) 
+		new_desk = Scr.CurrentDesk ;
+    
+	if( !force && ASWIN_DESK(asw) == new_desk )
         return ;
 
     old_desk = ASWIN_DESK(asw) ;
